#!/usr/bin/python

__author__="BU"


"""This script parses GO/KEGG output to be compatibe with input for circle_dat from GOplot R library"""

# Usage python exon.order.py -i input.gtf -o output.tsv

from optparse import OptionParser
import gzip
from os import sys
import getopt
import locale
from functools import cmp_to_key
locale.setlocale(locale.LC_ALL, 'en_US.UTF-8')
import re


# ===============
## OPTION PARSING
# ===============

parser = OptionParser()
parser.add_option("-i", 
	help="The input format is output of GO or KEGG enrichment (no column names)")
parser.add_option('-o', help='GO/KEGG enrichment output compatible with circle_dat from GOplot R library')
parser.add_option('-g', help='Tab separated file with ensebmle gene ids and GO/KEGG terms (ourput generated using --genes)')
parser.add_option('-c', help='GO Category BP|MF|CC. For KEGG put KEGG')
options, args = parser.parse_args()

# Store input and output file

infile=''
outfile=''
genes=''

# Read command line args
myopts, args = getopt.getopt(sys.argv[1:],"i:o:g:c:")

if myopts == []:
    print("Usage: %s -i input -o output" % sys.argv[0])

for o, a in myopts:
    if o == '-i':
        infile=a
    if o == '-o':
        outfile=a
    if o == '-g':
        genes=a
    if o == '-c':
        categ=a


print ("Input file : %s \nOutput file: %s  \nGenes: %s\nCategory %s" % (infile,outfile,genes,categ))

# subtract

def parse():

    "This function slices genome sequence (in one line format) into fragments of length defined by the user. It also prints out the coordinates (start, stop, strand) for each fragment."
    goterm={}
    gopval={}
    genicstuff={}

    enrichment=open(infile, 'r')
    for line in enrichment:
        record=line.split('\n')[0].split('\t')
        #print record
        go=record[0]
        pval=record[1]
        term=record[6]
        #print term
        goterm[go]=term
        gopval[go]=pval 
        #print go, goterm[go] 
        #print go, gopval[go]

    gen=open(genes, 'r')
    for liner in gen:
        recorder=liner.split('\n')[0].split('\t')
        if recorder[1] != "gene_id":
            goID=recorder[0]    
            geneID=recorder[1]
            if goID not in genicstuff:
                genicstuff[goID]=[geneID]
            else:
                genicstuff[goID].append(geneID)  
                #print goID, genicstuff[goID] 

    f=open(outfile, 'w')
    header="category"+"\t"+"ID"+"\t"+"term"+"\t"+"genes"+"\t"+"adj_pval"
    f.write('%s\n' % header)
    for key in goterm:
        genestr=', '.join(map(str, genicstuff[key]))
        #print key, genestr
        f.write('%s\t' % categ)
        f.write('%s\t' % key)
        f.write('%s\t' % goterm[key])
        f.write('%s\t' % genestr)
        f.write('%s\n' % gopval[key])
    f.close()
           
   
parse()

