#!/usr/bin/python

__author__="BUR"


"""This script will help to further analyze the clean.strand.py output. The goal is to get the region id and its length per strand. Lengths for non-uniq ids will be summed."""

# Usage python exon.order.py -i input.gtf -o output.tsv

from optparse import OptionParser
import gzip
from os import sys
import getopt
import locale
from functools import cmp_to_key
locale.setlocale(locale.LC_ALL, 'en_US.UTF-8')
import re


# ===============
## OPTION PARSING
# ===============

parser = OptionParser()
parser.add_option("-i", 
	help="The input format is a bed file with ids of merged regions in the 4th field. This script is designed to further analyze the clean.strand.py output.)")
parser.add_option('-o', help='Tab-separated file with id and length of the region. Lengths are going to be summed per ids')
options, args = parser.parse_args()

# Store input and output file

infile=''
outfile=''

# Read command line args
myopts, args = getopt.getopt(sys.argv[1:],"i:o:")

if myopts == []:
    print("Usage: %s -i input -o output" % sys.argv[0])

for o, a in myopts:
    if o == '-i':
        infile=a
    if o == '-o':
        outfile=a

print ("Input file : %s \nOutput file: %s" % (infile,outfile))

def filter():
    
    #records={}
    strander={}
    lths={}

    bed=open(infile, 'r')
    for line in bed:
        #print "LINE", line
        line_split=line.split('\n')[0].split('\t')

        start=line_split[1]
        stop=line_split[2]
        name=line_split[3]
        #print "Name", name

        lth=int(stop)-int(start)
        #print "Length", lth 

        if name not in lths:
            lths[name]=[lth]
        else:
            lths[name].append(lth)   
 
        #print "OUT", name, lengths[name]   

        for elem in name.split(','):
            #print name,'\t',elem.split('=')[3]
            strand=elem.split('=')[3]
                

            if name not in strander:
                strander[name]=[strand]
            else:
                strander[name].append(strand)



    # PLUS STRAND
    plus=outfile+".plus"+".tsv"
    f=open(plus, "w")
    for key in lths:
        #if len(lths[key]) > 1:
        merged=sum(lths[key])
        #else:
        #merged=lths[key][0] 
        #print "MERGED", merged
        uniq=set(strander[key])
        #print "UNIQ", uniq 
        if list(uniq)[0] == "+": 
              
            f.write('%s\t' % key)
            f.write('%s\n' % merged)
    f.close()            


    # MINUS STRAND
    minus=outfile+".minus"+".tsv"
    f=open(minus, "w")
    for key in lths:
        #if len(lths[key]) > 1:
        merged=sum(lths[key])
        #else:
        #merged=lths[key][0] 
        #print "MERGED", merged
        uniq=set(strander[key])
        #print "UNIQ", uniq 
        if list(uniq)[0] == "-": 
              
            f.write('%s\t' % key)
            f.write('%s\n' % merged)
    f.close()            

    # ITERGENIC
    intergenic=outfile+".intergenic"+".tsv"
    f=open(intergenic, "w")
    for key in lths:
        #if len(lths[key]) > 1:
        merged=sum(lths[key])
        #else:
        #merged=lths[key][0] 
        #print "MERGED", merged
        uniq=set(strander[key])
        #print "UNIQ", uniq 
        if list(uniq)[0] == "S": 
              
            f.write('%s\t' % key)
            f.write('%s\n' % merged)
    f.close()            
 
filter()

        #if name not in records:
        #    records[name]=[line]
        #else:
        #    records[name].append(line)

