#!/usr/bin/python

__author__="BU"


"""This script selects dominant transcripts for a given gene based on its expression threshold and threshold of fold change between dominant transcript and next highly expressed transcript."""

# Usage python dominant.transcript.analysis.py -i FLUXoutput.gtf -o output.txt -rpkm N -FC 

from os import sys
import getopt
import heapq
from optparse import OptionParser

# ----------------
## OPTION PARSING
# ----------------

parser = OptionParser()
parser.add_option("-i",  help="The input format is tab-delimited BED file containing exon_id and trans_id")
parser.add_option("-o",  help='Output file')
parser.add_option("-p", help="PhastCons or PhyloP wigFix file")

options, args = parser.parse_args()


# Store input and output file

infile=''
outfile=''
wigfix=''

# Read command line args
myopts, args = getopt.getopt(sys.argv[1:],"i:o:p:")

# Loop over arguments and assign input to infile and output to outfile 
# o == option
# a == argument passed to the o

#if myopts == []:
#    print("Usage: %s -i input -o output" % sys.argv[0])

for o, a in myopts:
    if o == '-i':
        infile=a
    if o == '-o':
        outfile=a
    if o == '-p':
        wigfix=a

print ("Input file: %s \nOutput file: %s \nWigFix: %s" % (infile,outfile,wigfix))

#-------------------------------------------------------------------------------------------------
# Defining Functions
#-------------------------------------------------------------------------------------------------

def wigfixReader():
   
    wigfix_hash={}
    count=[]

    wigfix_file=open(wigfix, 'r')
    for record in wigfix_file:
        if record.startswith('fixedStep'):
            chrW=record.split(' ')[1].split("=")[1]
            startW=int(record.split(' ')[2].split("=")[1])
            stepW=int(record.split(' ')[3].split("=")[1].split('\n')[0])  
            #print chrW, startW, stepW
            count=[]
            #print 'CHECK HAS BEEN RESET!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!'    
        else:            
            score=record.split('\n')[0]
            pos=startW+len(count)
            count.append(pos)
              
            #if (pos,chrW) not in wigfix_hash:
            wigfix_hash[pos,chrW]=score 
            #else:
            #    wigfix_hash[pos,chrW].append(score) 

            #print pos,chrW, wigfix_hash[pos,chrW]
        
    return wigfix_hash

    
def bedReader(cons_hash):

    bed_hash={}
    trans_exon={}
    cons_score_hash={}
    trans_cons_score={}
    trans_av_cons_score={}

    bed_file=open(infile, 'r')
    for line in bed_file:
        line_split=line.split('\t')
        ch=line_split[0]
        start=line_split[1]
        stop=line_split[2]
        exon_id=line_split[4]
        trans_id=line_split[5].split('\n')[0]
        #print "chr", ch, "start", start, "stop", stop, "exon_id", exon_id, "trans_id", trans_id        
        #print "start", start, "stop", stop

        if (exon_id, ch) not in bed_hash:
            bed_hash[exon_id, ch]=[range(int(start), int(stop)+1)]
        else:
            bed_hash[exon_id, ch].append(range(int(start), int(stop)+1))

        #print "start", start, "stop", stop 
        #print exon_id, ch, bed_hash[exon_id, ch]
        
        if trans_id not in trans_exon:
            trans_exon[trans_id]=[exon_id]
        else:
            trans_exon[trans_id].append(exon_id)
    #print trans_exon   

    # WIGFIX SECTION

    wigfix_records=cons_hash
   
    for key1, key2 in bed_hash:
        #print "key1", key1
        #print "key2", key2
        for value in bed_hash[key1, key2]:
            for elem in value:
                #print elem
                if (elem, key2) in wigfix_records:
                    #print "YES"
                    if key1 not in cons_score_hash:
                        cons_score_hash[key1]=[float(wigfix_records[elem, key2])]
                    else:
                        cons_score_hash[key1].append(float(wigfix_records[elem, key2]))
                    #print key1, cons_score_hash[key1]

    for key in trans_exon:
        for item in trans_exon[key]:
            #print item
            if item in cons_score_hash:
                #print item, cons_score_hash[item]
                if key not in trans_cons_score:
                    trans_cons_score[key]=[sum(cons_score_hash[item])/len(cons_score_hash[item])]
                else:
                    trans_cons_score[key].append(sum(cons_score_hash[item])/len(cons_score_hash[item]))
            #print key, trans_cons_score[key]                          

    for k in trans_cons_score:
            if k not in trans_av_cons_score:
                trans_av_cons_score[k]=[sum(trans_cons_score[k])/len(trans_cons_score[k])]
            else:
                trans_av_cons_score[k].append(sum(trans_cons_score[k])/len(trans_cons_score[k]))
            #print k, trans_av_cons_score[k]

    f=open(outfile, 'w')
    for kz in trans_av_cons_score:
        for vz in trans_av_cons_score[kz]:
            f.write('%s\t' % kz) 
            f.write('%s\n' % vz)          
    f.close()

     
cons_score=wigfixReader()
bedReader(cons_score)


