#!/usr/bin/python

__author__="BU"


"""This script selects dominant transcripts for a given gene based on its expression threshold and threshold of fold change between dominant transcript and next highly expressed transcript."""

# Usage python dominant.transcript.analysis.py -i FLUXoutput.gtf -o output.txt -rpkm N -FC 

from os import sys
import getopt
import heapq
from optparse import OptionParser

# ----------------
## OPTION PARSING
# ----------------

parser = OptionParser()
parser.add_option("-i", 
	help="The input format is flux output in .gtf file. No stdin")
parser.add_option('-o', help='Output file')
parser.add_option("-f", help="Dominance factor")
parser.add_option('-r',help="RPKM threshold")

options, args = parser.parse_args()


# Store input and output file

infile=''
outfile=''
rpkm_value=''
fold_change=''
filtered_file=''
duplicates=''

# Read command line args
myopts, args = getopt.getopt(sys.argv[1:],"i:o:r:f:")

# Loop over arguments and assign input to infile and output to outfile 
# o == option
# a == argument passed to the o

#if myopts == []:
#    print("Usage: %s -i input -o output" % sys.argv[0])

for o, a in myopts:
    if o == '-i':
        infile=a
    if o == '-o':
        outfile=a
    if o == '-r':
       rpkm_value=a
    if o == '-f':
       fold_change=a
    #if o == '-t':
    #   filtered_file=a

print ("Input file: %s \nOutput file: %s \nRPKM threshold: %s \nFC threshold: %s" % (infile,outfile,rpkm_value,fold_change))

#-------------------------------------------------------------------------------------------------
# Defining Functions
#-------------------------------------------------------------------------------------------------
                   
def filter_out():

    """ This function performes preliminary filtration of gtf file based on provided RPKM threshold. This is to control the output and deal with problem of dividing by 0, in case when one transcript is highly expressed and the second not expressed at all""" 
    filtered={}

    input_file=open(infile, 'r')
    for lines in input_file:
        lines_split=lines.split('\t')
        gtf_cigar=lines_split[8].split(';')

        for unit in gtf_cigar:
            #print unit
        
            if unit.startswith(' gene_id'):
                keygene=unit.split('"')[1]
                #print "keygene", keygene
            if unit.startswith('transcript_id'):
                keytrans=unit.split('"')[1]
                #print "keytrans", keytrans
            if unit.startswith(' RPKM'):  
                rpkm_score=unit.split(' ')[2].split('\n')[0]
                #print rpkm_score
            
                if float(rpkm_score) > float(rpkm_value):
                    if keygene not in filtered:
                        filtered[keygene]=[lines.split('\n')[0]]
                    else:
                        filtered[keygene].append(lines.split('\n')[0])
                        #print filtered    

    return filtered

#    f=open(filtered_file, 'w')
#    for k in filtered:
#        for v in filtered[k]:
#            for r in v:
#               f.write('%s' % r)
#    f.close()
      
def dominant_transcript(filtered_dict):
    """ Selecting dominant transcript based on RPKM and dominance factor"""    

    gene_trans={}
    trans_exp={}
    rpkm_set=[]
    dominants={}

    filtered_output=filtered_dict
    
    for keys in filtered_output:

        for values in filtered_output[keys]:
            #print values
            record_split=values.split('\t')
            #print record_split
            cigar=record_split[8].split(';')
            #print cigar

            for elem in cigar:
            
                if elem.startswith(' gene_id'):
                    gene_id=elem.split('"')[1]
                if elem.startswith('transcript_id'):
                    trans_id=elem.split('"')[1]
                if elem.startswith(' RPKM'):
                    rpkm=elem.split(' ')[2].split('\n')[0]

                    if gene_id not in gene_trans:
                        gene_trans[gene_id]=[trans_id]
                    else:
                        gene_trans[gene_id].append(trans_id)

                    if trans_id not in trans_exp:
                        trans_exp[trans_id]=[float(rpkm)]
                    else: 
                        trans_exp[trans_id].append(float(rpkm)) 

    for gene in gene_trans:
        trans_per_locus=gene_trans[gene]

        for trans in trans_per_locus:
            rpkm_set.append(trans_exp[trans][0])
  
        highly_expressed=heapq.nlargest(2, rpkm_set)
        highly_expressed.sort(reverse=True)
        if float(highly_expressed[0]) >= float(rpkm_value):
            #print 'HE', highly_expressed
            #print 'HE[0]', highly_expressed[0]
            if len(highly_expressed) == 1:
                position=rpkm_set.index(highly_expressed[0])
                dominants[gene]=(trans_per_locus[position],rpkm_set[position])
            if len(highly_expressed) > 1:
                if float(highly_expressed[0])/float(highly_expressed[1]) >= float(fold_change):
                    position=rpkm_set.index(highly_expressed[0])
                    position2=rpkm_set.index(highly_expressed[1])
                    dominants[gene]=(trans_per_locus[position],rpkm_set[position], rpkm_set[position2])

        rpkm_set=[]
    
    out=open(outfile, 'w')
    for g in dominants:
        out.write('%s\t' % g) 
        for t in dominants[g]:
            out.write('%s\t' % t)
        out.write('\n')
    out.close()        

#-------------------------------------------------------------------------------------------------
# EXECUTING
#-------------------------------------------------------------------------------------------------

filter_dict=filter_out()
dominant_transcript(filter_dict)




