#!/usr/bin/python

__author__="BU"


"""This script selects dominant transcripts for a given gene based on its expression threshold and threshold of fold change between dominant transcript and next highly expressed transcript."""

# Usage python dominant.transcript.analysis.py -i FLUXoutput.gtf -o output.txt -rpkm N -FC 

from os import sys
import getopt
import heapq

# Store input and output file

infile=''
outfile=''
rpkm_value=''
fold_change=''
filtered_file=''
duplicates=''

# Read command line args
myopts, args = getopt.getopt(sys.argv[1:],"i:o:r:f:t:d:")

# Loop over arguments and assign input to infile and output to outfile 
# o == option
# a == argument passed to the o

#if myopts == []:
#    print("Usage: %s -i input -o output" % sys.argv[0])

for o, a in myopts:
    if o == '-i':
        infile=a
    if o == '-o':
        outfile=a
    if o == '-r':
       rpkm_value=a
    if o == '-f':
       fold_change=a
    if o == '-t':
       filtered_file=a
    if o == '-d':
       duplicates=a

print ("Input file: %s \nOutput file: %s \nRPKM threshold: %s \nFC threshold: %s \nFiltered file: %s \nList of duplicated transcripts %s" % (infile,outfile,rpkm_value,fold_change,filtered_file, duplicates) )

#-------------------------------------------------------------------------------------------------
# Defining Functions
#-------------------------------------------------------------------------------------------------

def selecting_duplicate():
    
    tmp={} 
    selected={}
    duplist=[]     

    input_file=open(infile, 'r')
    for lines in input_file:
        lines_split=lines.split('\t')
        trans_ensembl=lines_split[0]  
        if trans_ensembl not in tmp:
            tmp[trans_ensembl]=[lines]
        else:
            tmp[trans_ensembl].append(lines) 
                
    duplicated_trans=open(duplicates, 'r')

    for duplicate in duplicated_trans:
        duplicate_split=duplicate.split('\n')[0]
        duplist.append(duplicate_split)
        

    for key in tmp:
        if key not in duplist:
            if key not in selected:
                selected[key]=[tmp[key]]
            else:
                selected[key].append(tmp[key])
                       
    return selected                      

def filter_out(prep_dict):

    filtered={}

    preprocessed=prep_dict
    for keys in preprocessed:
        
        for values in preprocessed[keys]:
            for record in values:
                record_split=record.split('\t')
                #print 'RECORD', record_split

                if record_split[0] != "tracking_id":
                                
                    keygene=record_split[3]
                    #print 'KEYGENE', keygene
                    keytrans=record_split[0]
                    #print 'KEYETRANS', keytrans
                    rpkm_score=record_split[9]
                    #print 'RPKM SCORE', rpkm_score 
                    if float(rpkm_score) > float(rpkm_value):
                        if keygene not in filtered:
                            filtered[keygene]=[record]
                        else:
                            filtered[keygene].append(record)
                            

    f=open(filtered_file, 'w')
    for k in filtered:
        for v in filtered[k]:
            for r in v:
               f.write('%s' % r)
    f.close()
       
def dominant_transcript():
    
    gene_trans={}
    trans_exp={}
    rpkm_set=[]
    dominants={}

    flux_output=open(filtered_file, 'r')
    
    for line in flux_output:
        line_split=line.split('\t')
        #print 'LS', line_split
         
        if line_split[0] != "tracking_id":
            
            gene_id=line_split[3]
            #print 'GENEID', '\n', gene_id
            trans_id=line_split[0]
            #print 'TRANS IS', '\n', trans_id
            rpkm=line_split[9]
            #print 'RPKM', '\n', rpkm

            if gene_id not in gene_trans:
                gene_trans[gene_id]=[trans_id]
            else:
                gene_trans[gene_id].append(trans_id)
            #print 'gene_trans', gene_id, gene_trans[gene_id]
            if trans_id not in trans_exp:
                trans_exp[trans_id]=[float(rpkm)]
            else: 
                trans_exp[trans_id].append(float(rpkm)) 
            #print 'TRANS ID', trans_id, trans_exp[trans_id]

    for gene in gene_trans:
        trans_per_locus=gene_trans[gene]

        for trans in trans_per_locus:
            rpkm_set.append(trans_exp[trans][0])
  
        highly_expressed=heapq.nlargest(2, rpkm_set)
        highly_expressed.sort(reverse=True)
        if float(highly_expressed[0]) >= float(rpkm_value):
            #print 'HE', highly_expressed
            #print 'HE[0]', highly_expressed[0]
            if len(highly_expressed) == 1:
                position=rpkm_set.index(highly_expressed[0])
                dominants[gene]=(trans_per_locus[position],rpkm_set[position])
            if len(highly_expressed) > 1:
                if float(highly_expressed[0])/float(highly_expressed[1]) >= float(fold_change):
                    position=rpkm_set.index(highly_expressed[0])
                    position2=rpkm_set.index(highly_expressed[1])
                    dominants[gene]=(trans_per_locus[position],rpkm_set[position], rpkm_set[position2])

        rpkm_set=[]
    
    out=open(outfile, 'w')
    for g in dominants:
        out.write('%s\t' % g) 
        for t in dominants[g]:
            out.write('%s\t' % t)
        out.write('\n')
    out.close()        

#-------------------------------------------------------------------------------------------------
# EXECUTING
#-------------------------------------------------------------------------------------------------

res_dict=selecting_duplicate()
filter_out(res_dict)
dominant_transcript()




