#!/usr/bin/python

__author__="BU"


"""This script selects dominant transcripts for a given gene based on its expression threshold and threshold of fold change between dominant transcript and next highly expressed transcript."""

# Usage python dominant.transcript.analysis.py -i FLUXoutput.gtf -o output.txt -rpkm N -FC 

from os import sys
import getopt
import heapq

# Store input and output file

infile=''
outfile=''
rpkm_value=''
fold_change=''
filtered_file=''
duplicates=''

# Read command line args
myopts, args = getopt.getopt(sys.argv[1:],"i:o:r:f:t:d:")

# Loop over arguments and assign input to infile and output to outfile 
# o == option
# a == argument passed to the o

#if myopts == []:
#    print("Usage: %s -i input -o output" % sys.argv[0])

for o, a in myopts:
    if o == '-i':
        infile=a
    if o == '-o':
        outfile=a
    if o == '-r':
       rpkm_value=a
    if o == '-f':
       fold_change=a
    if o == '-t':
       filtered_file=a
    if o == '-d':
       duplicates=a

print ("Input file: %s \nOutput file: %s \nRPKM threshold: %s \nFC threshold: %s \nFiltered file: %s \nList of duplicated transcripts %s" % (infile,outfile,rpkm_value,fold_change,filtered_file, duplicates) )

#-------------------------------------------------------------------------------------------------
# Defining Functions
#-------------------------------------------------------------------------------------------------

def selecting_duplicate():
    
    tmp={} 
    selected={}
    duplist=[]     

    input_file=open(infile, 'r')
    for lines in input_file:
        lines_split=lines.split('\t')
        gtf_cigar=lines_split[8].split(';')
 
        for element in gtf_cigar:
            if element.startswith('transcript_id'):
                list_trans_ensembl=element.split('"')[1].split('_')
                for trans_ensembl in list_trans_ensembl:
                    
                    
                    if trans_ensembl not in tmp:
                         tmp[trans_ensembl]=[lines]
                    else:
                         tmp[trans_ensembl].append(lines) 
                
    duplicated_trans=open(duplicates, 'r')

    for duplicate in duplicated_trans:
        duplicate_split=duplicate.split('\n')[0]
        duplist.append(duplicate_split)
        

    for key in tmp:
        if key not in duplist:
            if key not in selected:
                selected[key]=[tmp[key]]
            else:
                selected[key].append(tmp[key])
                       
    return selected                      

def filter_out(prep_dict):

    filtered={}

    preprocessed=prep_dict
    for keys in preprocessed:
        
        for values in preprocessed[keys]:
            
            for record in values:
                
                record_split=record.split('\t')
                core=record_split[8].split(';')
                

                for unit in core:
                    
                    if unit.startswith(' gene_id'):
                        keygene=unit.split('"')[1]
                        
                    if unit.startswith(' transcript_id'):
                        keytrans=unit.split('"')[1]
                        
                    if unit.startswith(' RPKM'):  
                        rpkm_score=unit.split(' ')[2].split('\n')[0]
                        
            
                        if float(rpkm_score) > float(rpkm_value):
                            if keygene not in filtered:
                                filtered[keygene]=[record]
                            else:
                                filtered[keygene].append(record)
                            

    f=open(filtered_file, 'w')
    for k in filtered:
        for v in filtered[k]:
            for r in v:
               f.write('%s' % r)
    f.close()
       
def dominant_transcript():
    
    gene_trans={}
    trans_exp={}
    rpkm_set=[]
    dominants={}

    flux_output=open(filtered_file, 'r')
    
    for line in flux_output:
        line_split=line.split('\t')
        
        cigar=line_split[8].split(';')
        

        for elem in cigar:
            
            if elem.startswith(' gene_id'):
                gene_id=elem.split('"')[1]
            if elem.startswith('transcript_id'):
                trans_id=elem.split('"')[1]
            if elem.startswith(' RPKM'):
                rpkm=elem.split(' ')[2].split('\n')[0]

                if gene_id not in gene_trans:
                    gene_trans[gene_id]=[trans_id]
                else:
                    gene_trans[gene_id].append(trans_id)

                if trans_id not in trans_exp:
                    trans_exp[trans_id]=[float(rpkm)]
                else: 
                    trans_exp[trans_id].append(float(rpkm)) 

    for gene in gene_trans:
        trans_per_locus=gene_trans[gene]

        for trans in trans_per_locus:
            rpkm_set.append(trans_exp[trans][0])
        highly_expressed=heapq.nlargest(2, rpkm_set)
        highly_expressed.sort(reverse=True)
        if float(highly_expressed[0]) > float(rpkm_value):
            if len(highly_expressed) == 1:
                position=rpkm_set.index(highly_expressed[0])
                dominants[gene]=(trans_per_locus[position],rpkm_set[position])
            if len(highly_expressed) > 1:
                if float(highly_expressed[0])/float(highly_expressed[1]) >= float(fold_change):
                    position=rpkm_set.index(highly_expressed[0])
                    position2=rpkm_set.index(highly_expressed[1])
                    dominants[gene]=(trans_per_locus[position],rpkm_set[position], rpkm_set[position2])

        rpkm_set=[]
    
    out=open(outfile, 'w')
    for g in dominants:
        out.write('%s\t' % g) 
        for t in dominants[g]:
            out.write('%s\t' % t)
        out.write('\n')
    out.close()        

#-------------------------------------------------------------------------------------------------
# EXECUTING
#-------------------------------------------------------------------------------------------------

res_dict=selecting_duplicate()
filter_out(res_dict)
dominant_transcript()




