#!/usr/bin/python

__author__="BU"


"""This script selects dominant transcripts for a given gene based on its expression threshold and threshold of fold change between dominant transcript and next highly expressed transcript."""

# Usage python dominant.transcript.analysis.py -i FLUXoutput.gtf -o output.txt -rpkm N -FC 

from os import sys
import getopt
import heapq
from optparse import OptionParser

# ----------------
## OPTION PARSING
# ----------------

parser = OptionParser()
parser.add_option("-i", 
	help="Three column tsv file containg gene id, transcript id and expression value. No stdin")
parser.add_option('-o', help='Output file')
parser.add_option("-f", help="Dominance factor")
parser.add_option('-r',help="RPKM threshold")

options, args = parser.parse_args()


# Store input and output file

infile=''
outfile=''
rpkm_value=''
fold_change=''

# Read command line args
myopts, args = getopt.getopt(sys.argv[1:],"i:o:r:f:")

# Loop over arguments and assign input to infile and output to outfile 
# o == option
# a == argument passed to the o

#if myopts == []:
#    print("Usage: %s -i input -o output" % sys.argv[0])

for o, a in myopts:
    if o == '-i':
        infile=a
    if o == '-o':
        outfile=a
    if o == '-r':
       rpkm_value=a
    if o == '-f':
       fold_change=a
    #if o == '-t':
    #   filtered_file=a

print ("Input file: %s \nOutput file: %s \nRPKM threshold: %s \nFC threshold: %s" % (infile,outfile,rpkm_value,fold_change))

#-------------------------------------------------------------------------------------------------
# Defining Functions
#-------------------------------------------------------------------------------------------------
                   
def filter_out():

    """ This function performes preliminary filtration of the tsv file based on provided RPKM threshold. This is to control the output and deal with problem of dividing by 0, in case when one transcript is highly expressed and the second not expressed at all""" 
    filtered={}

    input_file=open(infile, 'r')
    for lines in input_file:
        lines_split=lines.split('\t')
        keygene=lines_split[0]
        keytrans=lines_split[1]
        rpkm_score=lines_split[2].split('\n')[0]
        if float(rpkm_score) > float(rpkm_value):
            if keygene not in filtered:
                filtered[keygene]=[lines.split('\n')[0]]
            else:
                filtered[keygene].append(lines.split('\n')[0])
                
    return filtered

      
def dominant_transcript(filtered_dict):
    """ Selecting dominant transcript based on RPKM and dominance factor"""    

    gene_trans={}
    trans_exp={}
    rpkm_set=[]
    dominants={}

    filtered_output=filtered_dict
    
    for keys in filtered_output:

        for values in filtered_output[keys]:
            record_split=values.split('\t')
            gene_id=record_split[0]
            trans_id=record_split[1]
            rpkm=record_split[2].split('\n')[0]

            if gene_id not in gene_trans:
                gene_trans[gene_id]=[trans_id]
            else:
                gene_trans[gene_id].append(trans_id)

            if trans_id not in trans_exp:
                trans_exp[trans_id]=[float(rpkm)]
            else: 
                trans_exp[trans_id].append(float(rpkm)) 

    for gene in gene_trans:
        trans_per_locus=gene_trans[gene]

        for trans in trans_per_locus:
            rpkm_set.append(trans_exp[trans][0])
  
        highly_expressed=heapq.nlargest(2, rpkm_set)
        highly_expressed.sort(reverse=True)
        if float(highly_expressed[0]) >= float(rpkm_value):
            #print 'HE', highly_expressed
            #print 'HE[0]', highly_expressed[0]
            if len(highly_expressed) == 1:
                position=rpkm_set.index(highly_expressed[0])
                dominants[gene]=(trans_per_locus[position],rpkm_set[position])
            if len(highly_expressed) > 1:
                if float(highly_expressed[0])/float(highly_expressed[1]) >= float(fold_change):
                    position=rpkm_set.index(highly_expressed[0])
                    position2=rpkm_set.index(highly_expressed[1])
                    dominants[gene]=(trans_per_locus[position],rpkm_set[position], rpkm_set[position2])

        rpkm_set=[]
    
    out=open(outfile, 'w')
    for g in dominants:
        out.write('%s\t' % g) 
        for t in dominants[g]:
            out.write('%s\t' % t)
        out.write('\n')
    out.close()        

#-------------------------------------------------------------------------------------------------
# EXECUTING
#-------------------------------------------------------------------------------------------------

filter_dict=filter_out()
dominant_transcript(filter_dict)




