#!/usr/bin/python

'''This program helps to get rid of duplicated primer sequences and also reports the ids of the primers assigned to the same sequences. It was created for primer design of the primers that failed the validation'''

__author__="BU"

from optparse import OptionParser
import gzip
from os import sys
import getopt

# ===============
## OPTION PARSING
# ===============
parser = OptionParser()
parser.add_option("-i", 
        help="The input format is tsv file with primer id and its sequence. No stdin")
parser.add_option('-o', help='Output file to convert to fasta')
parser.add_option('-r', help='Output file with duplicated ids and primer seq')
options, args = parser.parse_args()


# Store input and output file

infile=''
outfile=''
report=''

# Read command line args
myopts, args = getopt.getopt(sys.argv[1:],"i:o:r:")

# Loop over arguments and assign input to infile and output to outfile 
# o == option
# a == argument passed to the o

if myopts == []:
    print("Usage: %s -i input -o output -m mode" % sys.argv[0])

for o, a in myopts:
    if o == '-i':
        infile=a
    if o == '-o':
        outfile=a
    if o == '-r': 
        report=a 
 
print ("Input file : %s \nOutput file: %s\nDuplicates: %s" % (infile,outfile,report))

# ========================
# Body of the function
# ========================

def getPrimers():

    primers={}
    duplicates={}

    gtf_file=open(infile, 'r')
    for line in gtf_file:
        line_split=line.split('\n')[0].split('\t')
        primerID=line_split[0]
        primerSEQ=line_split[1]

        primers[primerSEQ]=primerID

        if primerSEQ not in duplicates:
            duplicates[primerSEQ]=[primerID]
        else:
            duplicates[primerSEQ].append(primerID)

        
    f=open(outfile, "w")
    for key in primers:
        f.write('%s\t' % primers[key])
        f.write('%s\n' % key)
    f.close() 

    
    fil=open(report, "w")
    for kyz in duplicates:
        if len(duplicates[kyz]) > 1:
            for v in duplicates[kyz]:  
                fil.write('%s\t' % v)
            fil.write('%s\n' % kyz)
    fil.close() 


getPrimers()  
        
