#!/usr/bin/python

__author__="BU"

from optparse import OptionParser
import gzip
from os import sys
import getopt

# ===============
## OPTION PARSING
# ===============

parser = OptionParser()
parser.add_option("-i", 
        help="GTF/GTF. No stdin")
parser.add_option('-o', help='Output GTF file with intron edges')
parser.add_option('-r', help='Size of the genomic region')
options, args = parser.parse_args()


# Store input and output file

infile=''
outfile=''
reg=''

# Read command line args
myopts, args = getopt.getopt(sys.argv[1:],"i:o:r:")

# Loop over arguments and assign input to infile and output to outfile 
# o == option
# a == argument passed to the o

if myopts == []:
    print("Usage: %s -i input -o output -r reg " % sys.argv[0])

for o, a in myopts:
    if o == '-i':
        infile=a
    if o == '-o':
        outfile=a
    if o == '-r':
        reg=a
       
print ("Input file : %s \nOutput file: %s \nRegion: %s" % (infile,outfile,reg))


def split_introns():

    final=[]
    records={}
    strt={}
    stp={}    

    gtf_file=open(infile, 'r')
    for line in gtf_file:
        line_split=line.split("\n")[0].split("\t")
        typ=line_split[2]

        if typ == "intron" or typ == "exon":

           txID=line_split[8].split("\n")[0].split('"')[3]

           if txID not in records:
               records[txID]=[line.split("\n")[0]]
           else:
               records[txID].append(line.split("\n")[0])
           
           #print txID, records[txID]

           start=line_split[3]  
           stop=line_split[4]
           
             
           # get exon starts
           if txID not in strt:
               strt[txID]=[int(start)]
           else:
               strt[txID].append(int(start))

           # get exon stops
           if txID not in stp:
               stp[txID]=[int(stop)]
           else:
               stp[txID].append(int(stop))


    for key in records:
        interStrt=sorted(strt[key][1:-1])
        interStp=sorted(stp[key][1:-1])

        firstStrt=sorted(strt[key])[0]
        firstStp=sorted(stp[key])[0]

        lastStrt=sorted(strt[key])[-1]
        lastStp=sorted(stp[key])[-1]
        
        #print key, firstStrt
        #print key, lastStrt
        #print "Starts", key, interStrt 
        #print "Stops", key, interStp
 
        for item in records[key]:
            #print item
           
            item_split=item.split("\t")
            chrN=item_split[0]
            sth=item_split[1]
            typ=item_split[2]
            startR=item_split[3]  
            stopR=item_split[4]
            dot1=item_split[5]
            strand=item_split[6]
            dot2=item_split[7]
            cigar=item_split[8]
            geneID=item_split[8].split('"')[1]
            geneTp=item_split[8].split('"')[5]

            #print cigar 
            #print geneID               
            #print geneTp

            if int(startR) in interStrt and int(stopR) in interStp:
                new_start=str(int(startR)-int(reg))
                new_stop=str(int(stopR)+int(reg))
                new_record=(chrN+"\t"+sth+"\t"+typ+"\t"+new_start+"\t"+new_stop+"\t"+dot1+"\t"+strand+"\t"+dot2+"\t"+"gene_id"+' "'+geneID+"_INT"+'"'+";"+" "+"transcript_id"+' "'+key+'"'+";"+" "+"gene_type"+' "'+geneTp+'"'+";")
                final.append(new_record)
 
            elif int(startR) == firstStrt and int(stopR) == firstStp:
                new_start=str(int(startR))
                new_stop=str(int(stopR)+int(reg))
                new_record=(chrN+"\t"+sth+"\t"+typ+"\t"+new_start+"\t"+new_stop+"\t"+dot1+"\t"+strand+"\t"+dot2+"\t"+"gene_id"+' "'+geneID+"_INT"+'"'+";"+" "+"transcript_id"+' "'+key+'"'+";"+" "+"gene_type"+' "'+geneTp+'"'+";")
                final.append(new_record)

            elif int(startR) == lastStrt and int(stopR) == lastStp:
                new_start=str(int(startR)-int(reg))
                new_stop=str(int(stopR))
                new_record=(chrN+"\t"+sth+"\t"+typ+"\t"+new_start+"\t"+new_stop+"\t"+dot1+"\t"+strand+"\t"+dot2+"\t"+"gene_id"+' "'+geneID+"_INT"+'"'+";"+" "+"transcript_id"+' "'+key+'"'+";"+" "+"gene_type"+' "'+geneTp+'"'+";")
                final.append(new_record)

                                
    f=open(outfile, "w")
    for k in final:
        f.write('%s\n' % k)
    f.close() 

split_introns()

'''
            elif int(startR) in firstStrt and int(stopR) in firstStp:
                new_start=str(int(startR))
                new_stop=str(int(stopR)+int(reg))
                new_record=(chrN+"\t"+sth+"\t"+typ+"\t"+new_start+"\t"+new_stop+"\t"+dot1+"\t"+strand+"\t"+dot2+"\t"+"gene_id"+' "'+geneID+"_INT"+'"'+";"+" "+"transcript_id"+' "'+txID+"_INT"+'"'+";"+" "+"gene_type"+' "'+geneTp+'"'+";")
                final.append(new_record)

            elif int(startR) in lastStrt and int(stopR) in lastStp:
                new_start=str(int(startR)-int(reg))
                new_stop=str(int(stopR))
                new_record=(chrN+"\t"+sth+"\t"+typ+"\t"+new_start+"\t"+new_stop+"\t"+dot1+"\t"+strand+"\t"+dot2+"\t"+"gene_id"+' "'+geneID+"_INT"+'"'+";"+" "+"transcript_id"+' "'+txID+"_INT"+'"'+";"+" "+"gene_type"+' "'+geneTp+'"'+";")
                final.append(new_record)
'''


'''
                chrN=line_split[0]
                sth=line_split[1]
                typ=line_split[2]
                start=line_split[3]  
                stop=line_split[4]
                dot1=line_split[5]
                strand=line_split[6]
                dot2=line_split[7]
                cigar=line_split[8].split("\n")[0]
                geneID=line_split[8].split("\n")[0].split('"')[1]
                txID=line_split[8].split("\n")[0].split('"')[3]
                geneTp=line_split[8].split("\n")[0].split('"')[5]

                
                #print txID
                #print cigar                
                #print geneTp
                length=int(stop)-int(start)
                thr=2*int(reg)
                #print length
                #print thr
                if length > 2*int(reg):
                    new_start=str(int(start)+int(reg))
                    new_stop=str(int(stop)-int(reg))
                    new_recordINT=(chrN+"\t"+sth+"\t"+typ+"\t"+new_start+"\t"+new_stop+"\t"+dot1+"\t"+strand+"\t"+dot2+"\t"+"gene_id"+' "'+geneID+"_INT"+'"'+";"+" "+"transcript_id"+' "'+txID+"_INT"+'"'+";"+" "+"gene_type"+' "'+geneTp+'"'+";")
                    interior.append(new_recordINT)
'''

