#!/usr/bin/python

__author__="BU"


"""This script investigates the intron retention. Checks which introns are retained and prints out the details into tsv file."""

# Usage python exon.order.py -i input.gtf -o output.tsv

from optparse import OptionParser
import gzip
from os import sys
import getopt
import locale
from functools import cmp_to_key
locale.setlocale(locale.LC_ALL, 'en_US.UTF-8')
import re

# ===============
## OPTION PARSING
# ===============

parser = OptionParser()
parser.add_option("-i", 
	help="The input format is sorted gtf file with exon records. No stdin")
parser.add_option('-r', help='Sorted gtf file with corresponding intron records')
parser.add_option('-o', help='Output is the tsv file with the exon details')
options, args = parser.parse_args()

# Store input and output file

infile=''
intronF=''
outfile=''

# Read command line args
myopts, args = getopt.getopt(sys.argv[1:],"i:r:o:")

if myopts == []:
    print("Usage: %s -i input -o output" % sys.argv[0])

for o, a in myopts:
    if o == '-i':
        infile=a
    if o == '-o':
        outfile=a
    if o == '-r':
        intronF=a

print ("Input file : %s \nOutput file: %s  \nIntron file: %s" % (infile,outfile,intronF) )

# Body of exon_order function 

def atof(text):
    try:
        retval = float(text)
    except ValueError:
        retval = text
    return retval

def natural_keys(text):
    '''
    alist.sort(key=natural_keys) sorts in human order
    http://nedbatchelder.com/blog/200712/human_sorting.html
    (See Toothy's implementation in the comments)
    float regex comes from https://stackoverflow.com/a/12643073/190597
    '''
    return [ atof(c) for c in re.split(r'[+-]?([0-9]+(?:[.][0-9]*)?|[.][0-9]+)', text) ]

def exon_order():

    exons_trans={}
    exOrder={}

    gtf=open(infile)
    for line in gtf:
       line_split = line.strip().split('\t')
       #print line_split
       record_type = line_split[2]
       if record_type.startswith('exon'):
           chrom=line_split[0]
           #print 'CHROM', chrom
           start=line_split[3]
           #print 'START', start
           stop=line_split[4]
           #print 'STOP', stop
           strand=line_split[6]
           #print 'STR', strand
           #print "element_set", line_split[9]
           element_set = line_split[8].strip().split(";")
            
           for element in element_set:
               if element.strip().split(" ")[0].startswith('transcript_id'):
                   trans_id=element.strip().split('"')[1]
                   #print 'TRANS ID', trans_id

                   if trans_id not in exons_trans:
                        exons_trans[trans_id]=[chrom+'_'+start+'_'+stop+'_'+strand]
                   else:
                       exons_trans[trans_id].append(chrom+'_'+start+'_'+stop+'_'+strand)
                   #print 'RESULTS',  trans_id, exons_trans[trans_id]
                   #print len(exons_trans.keys())

    for key in exons_trans:
         strd=exons_trans[key][0].split('_')[3]
         if strd == "+":
             exOrder[key]=sorted(exons_trans[key], key=natural_keys)
         if strd == "-":
             exOrder[key]=sorted(exons_trans[key], key=natural_keys, reverse=True) 
    return exOrder 

def intron_order():

    introns_trans={}
    intOrder={}
    
    intrGtf=open(intronF)
    for intrLine in intrGtf:
       intrLine_split = intrLine.strip().split('\t')
       #print intrLine_split
       intRecord_type = intrLine_split[2]
       if intRecord_type.startswith('intron'):
           intChrom=intrLine_split[0]
           #print 'CHROM', intChrom
           intStart=intrLine_split[3]
           #print 'START', intStart
           intStop=intrLine_split[4]
           #print 'STOP', intStop
           intStrand=intrLine_split[6]
           #print 'STR', intStrand
           #print "element_set", intrLine_split[9]
           intElement_set = intrLine_split[8].strip().split(";")

           for intElement in intElement_set:
               if intElement.strip().split(" ")[0].startswith('transcript_id'):
                   intTrans_id=intElement.strip().split('"')[1]
                   #print 'TRANS ID', intTrans_id

                   if intTrans_id not in introns_trans:
                        introns_trans[intTrans_id]=[intChrom+'_'+intStart+'_'+intStop+'_'+intStrand]
                   else:
                       introns_trans[intTrans_id].append(intChrom+'_'+intStart+'_'+intStop+'_'+intStrand)
                   #print 'RESULTS',  intTrans_id, introns_trans[intTrans_id]
                   #print len(introns_trans.keys())

    for kyz in introns_trans:
         intStrd=introns_trans[kyz][0].split('_')[3]
         if intStrd == "+":
             intOrder[kyz]=sorted(introns_trans[kyz], key=natural_keys)
         if intStrd == "-":
             intOrder[kyz]=sorted(introns_trans[kyz], key=natural_keys, reverse=True) 
    return intOrder

def IR():

    f=open(outfile, 'w')
    introns=intron_order()
    exons=exon_order()
    print "INTRONS", introns
    print "EXONS", exons
    for k in introns:
        for item in introns[k]:
            #print item
            pos=introns[k].index(item)
            posF="intron_" + str(1+pos)
            #print 'POS', pos
            strnd=item.split('_')[3]
            #print strnd
            intStartCoord=item.split('_')[1]
            print "INTRON start", intStartCoord
            intStopCoord=item.split('_')[2]
            print "INTRON stop", intStopCoord
            if k in exons:
                exStopCoord=exons[k][pos].split('_')[2]
                print "exon stop", exStopCoord
                exStartCoord=exons[k][pos].split('_')[1]
                print "exon start", exStartCoord
            if strnd == "+":
                if exStopCoord > intStartCoord and exStopCoord > intStopCoord:
                    typ="Retained"
                else:
                    typ="Not_retained"  
            if strnd == "-":
                if exStartCoord < intStartCoord and exStartCoord < intStopCoord:
                    typ="Retained"
                else:
                    typ="Not_retained"                     
           
            for elmnt in item.split('_'):
                #print elmnt
                f.write('%s\t' % elmnt)
            f.write('%s\t' % k)
            f.write('%s\t' % item)
            f.write('%s\t' % posF)
            f.write('%s\n' % typ)
    f.close()    

IR() 

