#!/usr/bin/python

__author__="BU"


"""This script investigates the intron retention. Checks which introns are retained and prints out the details into tsv file."""

# Usage python exon.order.py -i input.gtf -o output.tsv

from optparse import OptionParser
import gzip
from os import sys
import getopt
import locale
from functools import cmp_to_key
locale.setlocale(locale.LC_ALL, 'en_US.UTF-8')
import re
import pybedtools


# ===============
## OPTION PARSING
# ===============

parser = OptionParser()
parser.add_option("-i", 
	help="The input format is sorted gtf file with exon records. No stdin")
parser.add_option('-r', help='Sorted gtf file with corresponding intron records')
parser.add_option('-o', help='Output is the tsv file with the exon details')
options, args = parser.parse_args()

# Store input and output file

infile=''
intronF=''
outfile=''

# Read command line args
myopts, args = getopt.getopt(sys.argv[1:],"i:r:o:")

if myopts == []:
    print("Usage: %s -i input -o output" % sys.argv[0])

for o, a in myopts:
    if o == '-i':
        infile=a
    if o == '-o':
        outfile=a
    if o == '-r':
        intronF=a

print ("Input file : %s \nOutput file: %s  \nIntron file: %s" % (infile,outfile,intronF) )

# bedtools for python

def show_value(s):
     """
     Convert unicode to str under Python 2;
     all other values pass through unchanged
     """
     if sys.version_info.major == 2:
         if isinstance(s, unicode):
             return str(s)
     return s

def bedtools():

    bedtoolsOut=[]

    exon_gtf=pybedtools.BedTool(infile)
    #print exon_gtf
    intron_gtf=pybedtools.BedTool(intronF)
    #print intron_gtf    
    BTintersect=pybedtools.bedtool.BedTool(intron_gtf).intersect(b=exon_gtf, f=1, s=True, wa=True, wb=True)
    for linebt in BTintersect:
        #print "LINEBT", type(linebt)
        linebt_split=[show_value(i) for i in linebt.fields]
        #print "LINEBT_SPLIT", linebt_split
        #linebt_split=linebt.split('\t')
        intron_chr=linebt_split[0]
        intron_str=linebt_split[2]
        intron_start=linebt_split[3]
        intron_stop=linebt_split[4]
        intron_strand=linebt_split[6]
        elementint_set = linebt_split[8].strip().split(";")
        for elementint in elementint_set:
            if elementint.strip().split(" ")[0].startswith('transcript_id'):
                intron_transId=elementint.strip().split('"')[1]                 
                intron_info=intron_chr+"\t"+intron_str+"\t"+intron_start+"\t"+intron_stop+"\t"+intron_strand+"\t"+intron_transId
                         
        exon_chr=linebt_split[9]
        exon_str=linebt_split[11]
        exon_start=linebt_split[12]
        exon_stop=linebt_split[13]
        exon_strand=linebt_split[15]
        elementexon_set = linebt_split[17].strip().split(";")
        for elementexon in elementexon_set:
            if elementexon.strip().split(" ")[0].startswith('transcript_id'):
                exon_transId=elementexon.strip().split('"')[1]                 
                exon_info=exon_chr+"\t"+exon_str+"\t"+exon_start+"\t"+exon_stop+"\t"+exon_strand+"\t"+exon_transId
                full_info=exon_info+"\t"+intron_info
                bedtoolsOut.append(full_info)

    return bedtoolsOut
    f=open('bedtoolsOut.tmp.tsv', 'w') 
    for line in bedtoolsOut:
        f.write('%s\n' % line)
    f.close()

def exon_order():

    exons_trans={}
    retained={}
    order={}

    gtf=open(infile)
    for line in gtf:
       line_split = line.strip().split('\t')
       #print line_split
       record_type = line_split[2]
       if record_type.startswith('exon'):
           chrom=line_split[0]
           #print 'CHROM', chrom
           start=line_split[3]
           #print 'START', start
           stop=line_split[4]
           #print 'STOP', stop
           strand=line_split[6]
           #print 'STR', strand
           #print "element_set", line_split[9]
           element_set = line_split[8].strip().split(";")
            
           for element in element_set:
               if element.strip().split(" ")[0].startswith('transcript_id'):
                   trans_id=element.strip().split('"')[1]
                   #print 'TRANS ID', trans_id

                   if trans_id not in exons_trans:
                        exons_trans[trans_id]=[chrom+'_'+start+'_'+stop+'_'+strand]
                   else:
                       exons_trans[trans_id].append(chrom+'_'+start+'_'+stop+'_'+strand)
                   #print 'RESULTS',  trans_id, exons_trans[trans_id]
                   #print len(exons_trans.keys())


    ir=bedtools()
    for record in ir:
        record_split=record.split('\t')
        exonIRchr=record_split[0]
        exonIRstart=record_split[2]
        exonIRstop=record_split[3]
        exonIRstrand=record_split[4]
        exonIRtxID=record_split[5]

        exon_coord=exonIRchr+'_'+exonIRstart+'_'+exonIRstop+'_'+exonIRstrand 
        if exonIRtxID not in retained:
            retained[exonIRtxID]=[exon_coord]
        else:
            retained[exonIRtxID].append(exon_coord)
        #print 'RESULTS',  exonIRtxID, retained[exonIRtxID]

    f=open(outfile, 'w') 
    for key in retained:
        if key in exons_trans:
            strd=exons_trans[key][0].split('_')[3]
            if strd == "+":
                exon_list=sorted(exons_trans[key])
            if strd == "-":
                exon_list=sorted(exons_trans[key], reverse=True) 
            for item in set(retained[key]):
                total_exn=len(exon_list)
                pos=1 + exon_list.index(item)
                posF="exon_" + str(pos)
                total_exn="total_" +str(len(exon_list))
                for element in item.split('_'):
                    #print element
                    f.write('%s\t' % element)
                f.write('%s\t' % key)
                f.write('%s\t' % item)
                f.write('%s\t' % posF)
                f.write('%s\n' % total_exn)
    f.close()     
              
exon_order()

