#!/usr/bin/python

__author__="BU"


"""This script converts gff3 to gtf"""

# Usage python gff3Togtf.py -i annotation.gff3 -o output.gtf 

from os import sys
import getopt
import heapq
from optparse import OptionParser

# ----------------
## OPTION PARSING
# ----------------

parser = OptionParser()
parser.add_option("-i",  help="Annotation in gff")
parser.add_option("-o",  help='Output gtf file')

options, args = parser.parse_args()


# Store input and output file

infile=''
outfile=''

# Read command line args
myopts, args = getopt.getopt(sys.argv[1:],"i:o:")

# Loop over arguments and assign input to infile and output to outfile 
# o == option
# a == argument passed to the o

#if myopts == []:
#    print("Usage: %s -i input -o output" % sys.argv[0])

for o, a in myopts:
    if o == '-i':
        infile=a
    if o == '-o':
        outfile=a

print ("Input file: %s \nOutput file: %s" % (infile,outfile))

#-------------------------------------------------------------------------------------------------
# Defining Functions
#-------------------------------------------------------------------------------------------------

def gffTogtf():
   
    gtf={}
    tx2gene={}

    gff=open(infile, "r")
    for line in gff:
        if not line.startswith('#'):
            split=line.split("\n")[0].split('\t')
             
            if split[2]=="gene":
                #print 'Gene', split
                chrN=split[0]   
                add=split[1]
                typ=split[2]
                start=split[3]
                stop=split[4]
                dot1=split[5]
                strand=split[6]
                dot2=split[7]
                 
                spliter=split[8].split(";")
                #print spliter
                for item in spliter:           
                    if item.startswith('ID'): 
                        gene_id=item.split('=')[1].split('\n')[0]
                        #print "GENEID", gene_id
                       
                    if item.startswith('GENE_TYPE'): 
                        gene_biotype=item.split('=')[1].split('\n')[0]
                        #print "GENEBT", gene_biotype

                    if item.startswith('STATUS'): 
                        gene_status=item.split('=')[1].split('\n')[0]
                        #print "GENEST", gene_status    


                        record=(chrN+"\t"+add+"\t"+typ+"\t"+start+"\t"+stop+"\t"+dot1+"\t"+strand+"\t"+dot2+"\t"+"gene_id"+" "+'"'+gene_id+'"'"; "+"transcript_id"+" "+'"'+gene_id+'"'"; "+"gene_status"+" "+'"'+gene_status+'"'"; "+"gene_biotype"+" "+'"'+gene_biotype+'"'";")
           
                        if gene_id not in gtf:
                           gtf[gene_id]=[record]
                        else:
                           gtf[gene_id].append(record)
                        #print gtf[gene_id]


            if split[2]=="transcript":
                #print 'TX', split
                chrN=split[0]
                add=split[1]
                typ=split[2]
                start=split[3]
                stop=split[4]
                dot1=split[5]
                strand=split[6]
                dot2=split[7]
                  
                spliter=split[8].split(";")
                for item in spliter:
                    if item.startswith('ID'): 
                        transcript_id=item.split('=')[1].split('\n')[0]
                        #print transcript_id
                    if item.startswith('Parent'): 
                        parent_id=item.split('=')[1].split(',')[0]
                        tx2gene[transcript_id]=parent_id
                        #print gene_id
                    if item.startswith('STATUS'): 
                        transcript_status=item.split('=')[1].split('\n')[0]
                        #print transcript_status
                    if item.startswith('TYPE'): 
                        transcript_biotype=item.split('=')[1].split('\n')[0]
                    if item.startswith('ASSEMBLED'): 
                        transcript_biotype="assembled_"+item.split('=')[1].split('\n')[0]

                        #print transcript_biotype

                        record=(chrN+"\t"+add+"\t"+typ+"\t"+start+"\t"+stop+"\t"+dot1+"\t"+strand+"\t"+dot2+"\t"+"gene_id"+" "+'"'+parent_id+'"'"; "+"transcript_id"+" "+'"'+transcript_id+'"'"; "+"transcript_status"+" "+'"'+transcript_status+'"'"; "+"transcript_biotype"+" "+'"'+transcript_biotype+'"'";")
           
                        if gene_id not in gtf:
                            gtf[gene_id]=[record]
                        else:
                            gtf[gene_id].append(record)     
                     
                            

            if split[2]=="exon":
                #print "EXON", split
                chrN=split[0]   
                add=split[1]
                typ=split[2]
                start=split[3]
                stop=split[4]
                dot1=split[5]
                strand=split[6]
                dot2=split[7] 

                spliter=split[8].split(";")
                for item in spliter:
                    if item.startswith('Parent'): 
                        exon_transcript_id=item.split('=')[1].split('\n')[0]
                        #print "exon_transcript_id", exon_transcript_id

                        exon_gene_id=tx2gene[exon_transcript_id]        
                        #print "exon_gene_id", exon_gene_id
               
                        record=(chrN+"\t"+add+"\t"+typ+"\t"+start+"\t"+stop+"\t"+dot1+"\t"+strand+"\t"+dot2+"\t"+"gene_id"+" "+'"'+exon_gene_id+'"'"; "+"transcript_id"+" "+'"'+exon_transcript_id+'"'"; "+"transcript_status"+" "+'"'+"NA"+'"'"; "+"transcript_biotype"+" "+'"'+"NA"+'"'";")
           
                        if gene_id not in gtf:
                            gtf[gene_id]=[record]
                        else:
                            gtf[gene_id].append(record)  
   
    f=open(outfile, "w")

    for key in gtf:
        for v in gtf[key]:
            f.write('%s\n' % v)
    f.close() 

gffTogtf()
