#!/usr/bin/python
"""This script reads FASTA input line by line and splices it into k-mers, where k is defined by user"""
"""It is a part of a genome fragmentation pipeline and should come after sequence split and deletion of palindromes and exons, just before reverse complement"""


from optparse import OptionParser
from os import sys
import getopt
import re
import numpy as np


parser = OptionParser()
parser.add_option("-i", help="FASTA file with sequences after palindrome removal")
parser.add_option("-k", help="Length")
parser.add_option("-o", help="Output file in FASTA format")

options, args = parser.parse_args()

sequenceinputfile=''
length=int()
outfile=''

myopts, args = getopt.getopt(sys.argv[1:],"i:k:o:")

if myopts == []:
    print("Usage: %s -i input -o output" % sys.argv[0])

for opt, arg in myopts:
    if opt == '-i':
        sequenceinputfile=arg
    if opt == '-k':
        length=int(arg)
    if opt == '-o':
        outfile=arg

# Load input files

# Sequence input
fasta=list()
with open(sequenceinputfile) as fastafile:
    for fastaline in fastafile:
        tmpline=fastaline.strip('\n')
        fasta.append(tmpline)
fastafile.close()


pieces = list()
records=open(sequenceinputfile, 'r')
for line in records:
    ID=line.strip('\n').split('\t')[0]
    sequence=line.strip('\n').split('\t')[1]
    #print line.strip('\n').split('\t')[0]
    #print sequence
    seg=np.arange(0, len(sequence), int(length))
    kmers=list()
    for i in seg:
        j=i+int(length)
        seqslice=sequence[i:j]
        if len(seqslice) == length:
            kmers.append(seqslice)
    splitseq=''
    for kmer in kmers:
        splitseq += kmer+'\t'
    record=ID+'\t'+splitseq
    with open(outfile, 'a+') as out:
        out.write('%s\n' % record)
