#!/usr/bin/python
"""This script reads FASTA input line by line and splices it into k-mers, where k is defined by user"""
"""It is a part of a genome fragmentation pipeline."""


from optparse import OptionParser
from os import sys
import getopt
import re
import numpy as np


parser = OptionParser()
parser.add_option("-i", help="Input file - 2 columns per line: ID and sequence")
parser.add_option("-k", help="Lenght of k-mer")
parser.add_option("-s", help="Sliding window step, default=1. Example: AAACCCGGGTTT, k=6,s=1 -> AAACCC AACCCG; s=2 -> AAACCC ACCCGG ")
parser.add_option("-o", help="Output file containing tab-separated ID and k-mers")

options, args = parser.parse_args()

sequenceinputfile=''
length=int()
sliding=1
outfile=''

myopts, args = getopt.getopt(sys.argv[1:],"i:k:s:o:")

if myopts == []:
    print("Usage: %s -i input -k kmer_length -s sliding_window -o output" % sys.argv[0])

for opt, arg in myopts:
    if opt == '-i':
        sequenceinputfile=arg
    if opt == '-k':
        length=int(arg)
    if opt == '-s':
        sliding=int(arg)
    if opt == '-o':
        outfile=arg

records=open(sequenceinputfile, 'r')
for line in records:
    ID=line.strip('\n').split('\t')[0]
    sequence=line.strip('\n').split('\t')[1]
    seg=np.arange(0, len(sequence), sliding)
    kmers=list()
    for i in seg:
        j=i+int(length)
        seqslice=sequence[i:j]
        if len(seqslice) == length:
            kmers.append(seqslice)
    splitseq=''
    for kmer in kmers:
        splitseq += kmer+'\t'
    record=ID+'\t'+splitseq
    with open(outfile, 'a+') as out:
        out.write('%s\n' % record)
