#!/usr/bin/python
"""This script creates a matrix of K-mer counts in records after palindrome removal"""

from optparse import OptionParser
from os import sys
import getopt
import re
import multiprocessing

parser = OptionParser()
parser.add_option("-i", help="Input file = output from palindromes, consisting of lines with seqID and tab-divided k-mers")
parser.add_option("-k", help="File with all unique kmers in file")
parser.add_option("-c", help="Number of CPUs for multiprocessing")
parser.add_option("-o", help="Output file - a matrix with all present k-mers and their counts in IDs")

options, args = parser.parse_args()

sequenceinputfile=''
kmerfile=''
outfile=''
mycpus=1

myopts, args = getopt.getopt(sys.argv[1:],"i:k:c:o:")

if myopts == []:
    print("Usage: %s -i input -k kmer_file -c cpu_number -o output" % sys.argv[0])

for opt, arg in myopts:
    if opt == '-i':
        sequenceinputfile=arg
    if opt == '-k':
        kmerfile=arg
    if opt == '-c':
        mycpus=int(arg)
    if opt == '-o':
        outfile=arg


# Read kmers
kmers=filter(None, sorted(set(open(kmerfile).read().split('\n'))))
# Make a header with K-mer sequences
header='####'
for kmer in kmers:
    header += '\t'+kmer
with open('header_for_matrix', 'w') as out:
    out.write('%s\n' % header)

# Worker function - scan line and print count of each k-mer from kmer_file, tab-separated
def mp_scanner(line):
    record=filter(None, line.strip('\n').split('\t'))
    newline = record[0]
    for kmer in kmers:
        newline += '\t'+str(line.count(kmer))
    return newline

# Multiprocessing tool
def mp_handler():
    # Safety check, 
    global mycpus
    cpus = multiprocessing.cpu_count()
    if mycpus > cpus:
        print "Error: Not enough CPUs. Greedy! Setting default - 1"
        mycpus = 1
    p = multiprocessing.Pool(mycpus)
    # Read input
    with open(sequenceinputfile) as f:
        linelist = [line.strip() for line in f]
    # Write output
    with open(outfile, 'w') as f:
        for result in p.imap(mp_scanner, linelist):
            f.write('%s\n' % result)

# Safety check for Windows - because of no "fork()", new processes would infinitely spawn. We don't want that.
if __name__=='__main__':
    mp_handler()
