#!/usr/bin/python
"""This script creates a matrix of K-mer counts in records after palindrome removal"""

from optparse import OptionParser
from os import sys
import getopt
import re
import multiprocessing
import itertools

parser = OptionParser()
parser.add_option("-i", help="Input file = output from palindromes, consisting of lines with seqID and tab-divided k-mers")
parser.add_option("-c", help="Number of CPUs for multiprocessing")
parser.add_option("-o", help="Output file - a matrix with all present k-mers and their counts in IDs")

options, args = parser.parse_args()

sequenceinputfile=''
outfile=''
mycpus=1

myopts, args = getopt.getopt(sys.argv[1:],"i:c:o:")

if myopts == []:
    print("Usage: %s -i input -c cpu_number -o output" % sys.argv[0])

for opt, arg in myopts:
    if opt == '-i':
        sequenceinputfile=arg
    if opt == '-c':
        mycpus=int(arg)
    if opt == '-o':
        outfile=arg


# Read unique kmers

#kmerset = filter(None, set(open(sequenceinputfile).read().strip('\n').split('\t')))
#kmers = sorted([item for item in kmerset if "PX" not in item])
#kmerset=''

# Make a header with K-mer sequences
kmerset=list(itertools.product('AGCT', repeat=6))
kmers=[]
header='####'

for kmer in kmerset:
    header += '\t'+''.join(kmer)
    kmers.append(''.join(kmer))  

#print kmers

def mp_scanner(line):
    record=filter(None, line.strip('\n').split('\t'))
    newline = record[0]
    for kmer in kmers:
        newline += '\t'+str(line.count(kmer))
    return newline

# Multiprocessing tool
def mp_handler():
    # Safety check, 
    global mycpus
    cpus = multiprocessing.cpu_count()
    if mycpus > cpus:
        print "Error: Not enough CPUs. Greedy! Setting default number of processes (1)."
        mycpus = 1
    p = multiprocessing.Pool(mycpus)
        
    # Read input and Write output
    with open(outfile, 'w') as f:
        global header
        f.write('%s\n' % header)
        for result in p.imap(mp_scanner, [line.strip() for line in open(sequenceinputfile, 'r')]):
            f.write('%s\n' % result)
            f.flush()
    f.close()

# Safety check for Windows - because of no "fork()", new processes would infinitely spawn. We don't want that.
if __name__=='__main__':
    mp_handler()

