#!/usr/bin/python
"""This script creates a matrix of K-mer counts in records after palindrome removal"""

from optparse import OptionParser
from os import sys
import getopt
import re
import multiprocessing

parser = OptionParser()
parser.add_option("-i", help="Input file = output from palindromes, consisting of lines with seqID and tab-divided k-mers")
parser.add_option("-o", help="Output file - a matrix with all present k-mers and their counts in IDs")

options, args = parser.parse_args()

sequenceinputfile=''
outfile=''
mycpus=1

myopts, args = getopt.getopt(sys.argv[1:],"i:o:")

if myopts == []:
    print("Usage: %s -i input -o output" % sys.argv[0])

for opt, arg in myopts:
    if opt == '-i':
        infile=arg
    if opt == '-o':
        outfile=arg


# Read unique kmers
kmerset = filter(None, set(open(infile).read().strip('\n').split('\t')))
kmers = sorted([item for item in kmerset if "PX" not in item])
kmerset=''

# Make a header with K-mer sequences
header='####'
for kmer in kmers:
    header += '\t'+kmer

# Worker function - scan line and print count of each k-mer from kmer_file, tab-separated
def mp_scanner():

    f=open(outfile, 'w')
    global header
    f.write('%s\n' % header)
    
    with open(infile) as pal:
        for line in pal:
            record=filter(None, line.strip('\n').split('\t'))
            newline = record[0]
            for kmer in kmers:
                newline += '\t'+str(line.count(kmer))
            f.write('%s\n' % newline)
            f.flush()
    f.close()            

# Safety check for Windows - because of no "fork()", new processes would infinitely spawn. We don't want that.
if __name__=='__main__':
    mp_scanner()
