#!/usr/bin/python

__author__="BU"

from optparse import OptionParser
import gzip
from os import sys
import getopt

# ===============
## OPTION PARSING
# ===============
parser = OptionParser()
parser.add_option("-i", 
	help="The input format is .txt file containing full names of all files that are going to be used for creating a metafile. No stdin")
parser.add_option("-c", help="Column number with identifier. All identifiers must be unique")
parser.add_option('-d',help="Header. Comma-separated items that are expected to be column names")
parser.add_option('-e',help="Column number with element for which metafile is going to be created")
parser.add_option('-o', help='Output file')
parser.add_option('-r',help='File containing reference identifiers. This is is required, when all input files (provided as a list in -i) have different number of rows')
parser.add_option('-s', help='Field separator(space or tab)')
options, args = parser.parse_args()


# Store input and output file

infile=''
outfile=''
ref_id=''
c_fld=''
r_fld=''
sep=''
header=''

# Read command line args
myopts, args = getopt.getopt(sys.argv[1:],"i:o:r:c:e:s:d:")

# Loop over arguments and assign input to infile and output to outfile 
# o == option
# a == argument passed to the o

if myopts == []:
    print("Usage: %s -i input -o output -m mode" % sys.argv[0])

for o, a in myopts:
    if o == '-i':
        infile=a
    if o == '-o':
        outfile=a
    if o == '-r':
        ref_id=a
    if o == '-c':
        c_fld=a
        c_use=int(c_fld)-1
    if o == '-e':
        r_fld=a
        r_use=int(r_fld)-1
    if o == '-d':
        header=a
    if o == '-s':
        sep=a
        if sep == "space":
            use=" "
        if sep == "tab":
            use='\t'    

print ("Input file : %s \nOutput file: %s \nReference ids: %s" % (infile,outfile,ref_id))

# ========================
# Body of GET_SJ function
# ========================
def get_metafile(gff_file):
     
    coord_line={}
   
    gff=open(gff_file, 'r')
    for line in gff:
        #print line
        line_split=line.strip().split(use)
        coord=line_split[c_use]
        rec=line_split[r_use]
        coord_line[coord]=rec
    
    return coord_line
    

def get_list(row_id_file):
    list_row_id=[]

    row_id=open(row_id_file, 'r')
    for l in row_id:
        l_split=l.strip().split('\n')
        #print 'L_SPLIT', '\n', l_split[0]
        list_row_id.append(l_split[0])
    return list_row_id       


meta_dict={}

list_of_files=open(infile, 'r')
for filename in list_of_files:
    #print 'FILENAME', '\n', filename
    f=filename.split('\n')[0]
    #print 'F', '\n', f
    file_dict=get_metafile(f)
    list_coord=get_list(ref_id)
    
    for elem in list_coord:
        if elem in file_dict:
            if elem not in meta_dict:
                meta_dict[elem]=[file_dict[elem]]
            else:
                meta_dict[elem].append(file_dict[elem])
        else:
            if elem not in meta_dict:
                meta_dict[elem]=['NA']
            else:
                meta_dict[elem].append('NA')

        #print 'META DICT', '\n', elem,  meta_dict[elem]
        #print 'META DICT ELEM 0', meta_dict[elem][0]
        #print 'META DICT ELEM 0', meta_dict[elem][0][0] 


f = open(outfile, 'w')

for it in header.split(','):
    f.write('%s\t' % it)
f.write('\n')
  
for key in meta_dict:
    f.write('%s\t' % key)
    for value in meta_dict[key]:
        f.write('%s\t' % value)
    f.write('\n')      
f.close()

