#!/usr/bin/python
"""This script checks which retained introns are from mitochondria"""

from optparse import OptionParser
from os import sys
import gzip
import getopt
import re

###

parser = OptionParser()
parser.add_option("-i", help="The input file in .tsv format with IR exon details (output from IR script")
parser.add_option("-m", help="The EnsemblGeneID column from MitoCarta2.0")
parser.add_option("-r", help=".gtf file with exon data")
parser.add_option("-o", help="Output file that looks the same as input .tsv, but has 2 additional columns with EnsemblGeneID and mito/non-mito information")

options, args = parser.parse_args()

tsvinputfile=''
mitocardfile=''
gtfinputfile=''
outfile=''

myopts, args = getopt.getopt(sys.argv[1:],"i:m:r:o:")

if myopts == []:
    print("Usage: %s -i input -o output" % sys.argv[0])

for opt, arg in myopts:
    if opt == '-i':
        tsvinputfile=arg
    if opt == '-m':
        mitocardfile=arg
    if opt == '-r':
        gtfinputfile=arg
    if opt == '-o':
        outfile=arg

print ("Input file : %s \nMitoCarta file : %s \nGtf file : %s \nOutput file : %s" % (tsvinputfile, mitocardfile, gtfinputfile, outfile) )

def mito_check():
    mito_genes=list()
    mitocard=open(mitocardfile)
    
    # Make list for mito/non-mito assignment
    for line in mitocard:
        tmpline=line.strip().split('\n')
        mito_genes.append(tmpline[0])
    #print mito_genes
    
    # Make a list with geneID-transID pair (to replace transID in .tsv input)
    
    gene_tx_pair=list()
    gtf=open(gtfinputfile)
    for line in gtf:
        line_split=line.strip().split('\t')
        #print line_split
        element_set = line_split[8].strip().split(";")
        for element in element_set:
            if element.strip().split(" ")[0].startswith('gene_id'):
                gtf_gene_ID=element.strip().split('"')[1]
            if element.strip().split(" ")[0].startswith('transcript_id'):
                gtf_trans_ID=element.strip().split('"')[1]
                gtf_join=gtf_gene_ID+"\t"+gtf_trans_ID
                gene_tx_pair.append(gtf_join)

    # Work on the .tsv file
    genetxjoin=list()

    input_IR=open(tsvinputfile)
    for line in input_IR:
        line_split=line.strip().split('\t')
        chromosome=line_split[0]
        seqstart=line_split[1]
        seqend=line_split[2]
        strand=line_split[3]
        tsv_trans_ID=line_split[4]
        tsvitem=line_split[5]
        posF=line_split[6]
        total_exn=line_split[7]
        for item in list(set(gene_tx_pair)):
            if tsv_trans_ID in item:
                status="non-mito"
                for mitoID in mito_genes:
                    if mitoID in str(item):
                        status="mito"
			break
		join=chromosome+'\t'+seqstart+'\t'+seqend+'\t'+strand+'\t'+item+'\t'+tsvitem+'\t'+posF+'\t'+total_exn+'\t'+status
                #print join
                genetxjoin.append(join)
		break
    #print genetxjoin
    f=open(outfile, 'w')
    for item in genetxjoin:
        f.write('%s\n' % item)
    f.close

mito_check()

