#!/usr/bin/python
"""This script checks if k-mers in input file are palindroms and, if they are, removes them from line"""

from optparse import OptionParser
from os import sys
import getopt
import re


parser = OptionParser()
parser.add_option("-i", help="FASTA file with genome nt sequence, no descriptions")
parser.add_option("-o", help="Output file consisting of n-sized lines, except palindromic ones")
parser.add_option("-k", help="Output file - a list of unique palindromes from -o")
options, args = parser.parse_args()

sequenceinputfile=''
outfile=''
uniquekmeroutfile=''
myopts, args = getopt.getopt(sys.argv[1:],"i:o:")

if myopts == []:
    print("Usage: %s -i input -o output" % sys.argv[0])

for opt, arg in myopts:
    if opt == '-i':
        sequenceinputfile=arg
    if opt == '-o':
        outfile=arg
    if opt == '-k':
        uniquekmeroutfile=arg

# Load the input file
records=list()
inputfile=open(sequenceinputfile)
for line in inputfile:
    tmpline=line.strip('\n')
    records.append(tmpline)
inputfile.close()

unique_kmers=list()
for line in records:
    record = filter(None, line.strip('').split('\t'))
    ID=record[0]
    nonpalindromes=''
    for kmer in record[1:]:
        N_count = kmer.count('N')
        if(kmer != kmer[::-1]) and N_count < 0.5*len(kmer):
            nonpalindromes += kmer+'\t'
            if kmer not in unique_kmers:
                unique_kmers.append(kmer)
    if nonpalindromes == '':
        continue
    newline=ID+'\t'+nonpalindromes
    with open(outfile, 'a+') as out:
        out.write('%s\n' % newline)
with open(uniquekmeroutfile) as kmout:
    for item in unique_kmers:
        kmout.write('%s\n' % item)


