#!/usr/bin/python
"""This script checks which retained introns belong to the secretome (endoplasmatic reticulum)"""

from optparse import OptionParser
from os import sys
import gzip
import getopt
import re

###

parser = OptionParser()
parser.add_option("-i", help="The input file in .tsv format (output from mito_check script")
parser.add_option("-s", help="The GeneID column from VerSeDa secretome data for given species")
parser.add_option("-o", help="Output file that looks the same as input .tsv, but has 2 additional columns with EnsemblGeneID and mito/non-mito information")

options, args = parser.parse_args()

tsvinputfile=''
secretomefile=''
outfile=''

myopts, args = getopt.getopt(sys.argv[1:],"i:s:o:")

if myopts == []:
    print("Usage: %s -i input -o output" % sys.argv[0])

for opt, arg in myopts:
    if opt == '-i':
        tsvinputfile=arg
    if opt == '-s':
        secretomefile=arg
    if opt == '-o':
        outfile=arg

print ("Input file : %s \nSecretome data file : %s \nOutput file : %s" % (tsvinputfile, secretomefile, outfile) )

def secretome_check():
    ER_genes=list()
    secretome=open(secretomefile)

    for line in secretome:
        tmpline=line.strip().split('\n')
        ER_genes.append(tmpline[0])

    ERjoin=list()
    input_mito=open(tsvinputfile)
    for line in input_mito:
        line_split=line.strip().split('\t')
        
        chromosome=line_split[0]
        seqstart=line_split[1]
        seqend=line_split[2]
        strand=line_split[3]
        tsv_gene_ID=line_split[4]
        tsv_trans_ID=line_split[5]
        tsvitem=line_split[6]
        posF=line_split[7]
        total_exn=line_split[8]
        mitostatus=line_split[9]

        ERstatus="non-ER"
        for item in ER_genes:
            if item in str(tsv_gene_ID):
                ERstatus="ER"
        
        join=chromosome+'\t'+seqstart+'\t'+seqend+'\t'+strand+'\t'+tsv_gene_ID+'\t'+tsv_trans_ID+'\t'+tsvitem+'\t'+posF+'\t'+total_exn+'\t'+mitostatus+'\t'+ERstatus
        ERjoin.append(join)
    #print genetxjoin
    f=open(outfile, 'w')
    for item in ERjoin:
        f.write('%s\n' % item)
    f.close

secretome_check()

