#!/usr/bin/python

__author__="BUR"


"""This script reduces the gencode gene biotypes in the 14th field of gtf"""

# Usage python exon.order.py -i input.gtf -o output.tsv

from optparse import OptionParser
import gzip
from os import sys
import getopt
import locale
from functools import cmp_to_key
locale.setlocale(locale.LC_ALL, 'en_US.UTF-8')
import re


# ===============
## OPTION PARSING
# ===============

parser = OptionParser()
parser.add_option("-i", 
	help="The input format is gtf file with gene biotype in 14th field)")
parser.add_option('-o', help='gtf file with reduced gene biotypes')
options, args = parser.parse_args()

# Store input and output file

infile=''
outfile=''

# Read command line args
myopts, args = getopt.getopt(sys.argv[1:],"i:o:")

if myopts == []:
    print("Usage: %s -i input -o output" % sys.argv[0])

for o, a in myopts:
    if o == '-i':
        infile=a
    if o == '-o':
        outfile=a

print ("Input file : %s \nOutput file: %s" % (infile,outfile))

gene_biotypes = {

    "3prime_overlapping_ncRNA" : "protein_coding",
    "antisense" : "lncRNA",
    "bidirectional_promoter_lncRNA" : "lncRNA",
    "IG_C_gene" : "protein_coding",
    "IG_C_pseudogene" : "pseudogene",
    "IG_D_gene" : "protein_coding",
    "IG_J_gene" : "protein_coding",
    "IG_J_pseudogene" : "pseudogene",
    "IG_D_pseudogene" : "pseudogene",
    "IG_pseudogene" : "pseudogene",
    "IG_V_gene" : "protein_coding",
    "IG_LV_gene" : "protein_coding",
    "IG_V_pseudogene" : "pseudogene",
    "lincRNA" : "lncRNA",
    "macro_lncRNA" : "lncRNA",
    "miRNA" : "non_coding",
    "misc_RNA" : "non_coding",
    "Mt_rRNA" : "non_coding",
    "Mt_tRNA" : "non_coding",
    "non_coding" : "non_coding",
    "polymorphic_pseudogene" : "pseudogene",
    "processed_pseudogene" : "pseudogene",
    "processed_transcript" : "lncRNA",
    "protein_coding" : "protein_coding",
    "pseudogene" : "pseudogene",
    "ribozyme" : "non_coding",
    "rRNA" : "non_coding",
    "scaRNA" : "non_coding",
    "scRNA" : "non_coding",
    "sense_intronic" : "lncRNA",
    "sense_overlapping" : "lncRNA",
    "snoRNA" : "non_coding",
    "snRNA" : "non_coding",
    "sRNA" : "non_coding",
    "TEC" : "protein_coding",
    "transcribed_processed_pseudogene" : "pseudogene",
    "transcribed_unitary_pseudogene" : "pseudogene",
    "transcribed_unprocessed_pseudogene" : "pseudogene",
    "translated_processed_pseudogene" : "pseudogene",
    "TR_C_gene" : "protein_coding",
    "TR_D_gene" : "protein_coding",
    "TR_J_gene" : "protein_coding",
    "TR_J_pseudogene" : "pseudogene",
    "TR_V_gene" : "protein_coding",
    "TR_V_pseudogene" : "pseudogene",
    "unitary_pseudogene" : "pseudogene",
    "unprocessed_pseudogene" : "pseudogene",
    "vaultRNA" : "non_coding",
    "lncRNA" : "lncRNA",
    "translated_unprocessed_pseudogene" : "pseudogene",
    "rRNA_pseudogene" : "pseudogene",
     "vault_RNA" : "non_coding"
         }

def exchange_gene_biotypes():

    gtf={}
    exchanged=[]

    gtf_file=open(infile, 'r')
    for line in gtf_file:
        line_split=line.split("\n")[0].split("\t")
        #print "LINE_SPLIT", line_split
        #typ=line_split[2]
        #if typ == "exon" or "intron" or "gene" or "transcript":
        chrN=line_split[0]
        sth=line_split[1]
        typ=line_split[2]
        start=line_split[3]  
        stop=line_split[4]
        dot1=line_split[5]
        strand=line_split[6]
        dot2=line_split[7]
        cigar=line_split[8].split("\n")[0].split(' ')
        gene_id=cigar[1].split('"')[1]
        trans_id=cigar[3].split('"')[1]
        gene_bio=cigar[5].split('"')[1]
        new_gene_bio=gene_biotypes[gene_bio]
        #print "CIGAR", cigar[5].split('"')[1]
        #print "new_gene_bio", new_gene_bio
        new_record=(chrN+"\t"+sth+"\t"+typ+"\t"+start+"\t"+stop+"\t"+dot1+"\t"+strand+"\t"+dot2+"\t"+"gene_id"+' "'+gene_id+'"'+";"+" "+"transcript_id"+' "'+trans_id+'"'+";"+" "+"gene_type"+' "'+new_gene_bio+'"'+";")
        #print "new_record", new_record
         
        exchanged.append(new_record) 

    f=open(outfile, "w")
    for k in exchanged:
        f.write('%s\n' % k)
    f.close() 
        
exchange_gene_biotypes()


