from trackhub import * 
import os
from trackhub.helpers import show_rendered_files
from trackhub.upload import upload_track, upload_hub

def makeTrackHub():

    url_base = 'http://genome.crg.es/~buszczynska/gencode/track.hub/suppSJ/data.files/'
    upload_base = '/users/rg/buszczynska/public_html/gencode/track.hub/suppSJ/data.files/'

# -----------------------------------------------------------
# HUB.TXT and GENOME FILE
# -----------------------------------------------------------

    hub = Hub(
        hub='suppSJ',
        short_label='GENCODE supported SJ',
        long_label='GENCODE SJ experimentaly validated or supported by RNA-Seq data (ENCODE and GTEx)',
        email='barbara.uszczynska@crg.es')

    genomes_file = GenomesFile()
    genome = Genome('hg19')
    trackdb = TrackDb()
    
# -----------------------------------------------------------
# GTEx SJ 
# -----------------------------------------------------------

    gtexSJ = CompositeTrack(
        name="GTExSJ",
        short_label="GTExSJ",
        long_label="SJ supported by GTEx",
        tracktype="bigBed 4")
   
    gtex_subgroups=[
 
    SubGroupDefinition(
        name="tissue",
        label="Tissue",
        mapping=dict(adipose="Adipose", artery="Artery", blood="Blood", brain="Brain", breast="Breast", colon="Colon", ebv="EBV", esophagus="Esophagus", fibroblasts="Fibroblasts", gland="Gland", heart="Heart", kidney="Kidney", liver="Liver", lung="Lung", muscle="Muscle", nerve="Nerve", ovary="Ovary", pancreas="Pancreas", pituitary="Pituitary", prostate="Prostate", skin="Skin", stomach="Stomach", testis="Testis", thyroid="Thyroid", tube="Tube", uterus="Uterus", vagina="Vagina", all="All"))]

    gtexSJ.add_subgroups(gtex_subgroups)
    gtexSJ.add_params(color='28,132,221')
    #gtexSJ.add_params(visibility='full') - Visibility has been defined in ViewTrack

    # GTEX View
    view_gtexSJ = ViewTrack(
        name="GTExSJ_VIEW",
        visibility="squish",
        view="Bed",
        tracktype="bigBed 4",
        short_label="GTExSJ_VIEW",
        long_label="SJ supported by GTEx VIEW")

    gtexSJ.add_view(view_gtexSJ)
    
    #print gtexSJ, '\n'
# -----------------------------------------------------------
# ENCODE SJ
# -----------------------------------------------------------

    encodeSJ = CompositeTrack(
        name="EncodeSJ",
        short_label="EncodeSJ",
        long_label="SJ supported by ENCODE",
        tracktype="bigBed 4")
       
    encode_subgroups=[
 
    SubGroupDefinition(
        name="cell",
        label="Cell",
        mapping=dict(A549="A549", AG04450="AG04450", BJ="BJ", CD20="CD20+", GM12878="GM12878",
H1hESC="H1-hESC", HAoAF="HAoAF", HAoEC="HAoEC", HAoSMC="HAoSMC", HBdMEC="HBdMEC", HBdSMC="HBdSMC", HBEpC="HBEpC", HBSMC="HBSMC", HCAEC="HCAEC", HCASMC="HCASMC", HCH="HCH", HCM="HCM", HDBEC="HDBEC", HDLEC="HDLEC", HDMEC="HDMEC", HeLaS3="HeLa-S3", HepG2="HepG2", HFDPC="HFDPC", hMSCAT="hMSC-AT", hMSCBM="hMSC-BM", hMSCUC="hMSC-UC", HNEpC="HNEpC", HOB="HOB", HPAEC="HPAEC",
HPCPL="HPC-PL", HPF="HPF", HPIEpC="HPIEpC", HPMEC="HPMEC", HRCEpC="HRCEpC", HSaVEC="HSaVEC", HSMM="HSMM", HTEpC="HTEpC", HUAEC="HUAEC", HUASMC="HUASMC", HUtSMC="HUtSMC", HUVEC="HUVEC", HVMF="HVMF", HWP="HWP", IMR90="IMR90", K562="K562", Liver="Liver", MCF7="MCF-7", MonoCD14="Monocytes-CD14+", NHDF="NHDF", NHEK="NHEK", NHEM_M2="NHEM_M2", NHLF="NHLF", SkMC="SkMC", SKNSH="SK-N-SH", SKNSHRA="SK-N-SH_RA", all="All"))]

    encodeSJ.add_subgroups(encode_subgroups) 
    encodeSJ.add_params(color='116,189,16')
    #print encodeSJ, '\n'

    # ENCODE_View
    view_encodeSJ = ViewTrack(
        name="EncodeSJ_View",
        visibility="squish",
        view="Bed",
        tracktype="bigBed 4",
        short_label="EncodeSJ_View",
        long_label="SJ supported by ENCODE VIEW")

    encodeSJ.add_view(view_encodeSJ)
   
# -----------------------------------------------------------
# EXPERIMENTAL VALIDATION
# -----------------------------------------------------------

    exp_bed="exp.validated.bb"
    basename=os.path.basename(exp_bed)
    expValSJ = Track(
        name="ExpValSJ",
        tracktype="bigBed 4",
        url=url_base + basename,
        visibility="squish",        
        short_label="ExpValSJ",
        long_label="SJ validated by RT-PCR-SEQ pipeline")

    expValSJ.add_params(color='234,194,92')

# -----------------------------------------------------------
# GTEX_ENCODE
# -----------------------------------------------------------

    gtex_encode_bed="gtex.encode.all.bb"
    basename=os.path.basename(gtex_encode_bed)
    gtexEncodeSJ = Track(
        name="GTExEncodeSJ",
        tracktype="bigBed 4",
        url=url_base + basename,
        visibility="squish",        
        short_label="GTExEncodeSJ",
        long_label="SJ supported by GTEx or ENCODE")

    gtexEncodeSJ.add_params(color='234,92,168')
# -----------------------------------------------------------
# ALL (GTEX/ENCODE/EXPVAL)
# -----------------------------------------------------------

    all_bed="all.bb"
    basename=os.path.basename(all_bed)
    allSJ = Track(
        name="allSuppSJ",
        tracktype="bigBed 4",
        url=url_base + basename,
        visibility="squish",        
        short_label="allSuppSJ",
        long_label="SJ experimentaly validated or are supported by GTEx/ENCODE data")

    allSJ.add_params(color='89,16,189')
 
# ----------------------------------------------------------
# Building tissue tracks for GTEx
# ----------------------------------------------------------
    gtex_inv={}
 
    for sg in gtex_subgroups:
        if sg.name == "tissue":
            for key in sg.mapping:
                gtex_inv[sg.mapping[key]]=key       
    
    gtex_tiss_file="/users/rg/buszczynska/Projects/sj.validation.180314/gen19.gtex.encode/gen19.gtex/gtex.tissues.txt"
 
    gtex_tiss=open(gtex_tiss_file, 'r')
    for tiss_tmp in gtex_tiss:
        tiss=tiss_tmp.split('\n')[0]
        gtex_bed="gtex."+tiss+".8.stg.reads.bb"
        basename=os.path.basename(gtex_bed)
        track = Track(
            name="GTExSJ"+"_"+tiss,
            tracktype='bigBed 4',
            url=url_base + basename,
            shortLabel='GTExSJ %s' % tiss,
            longLabel='SJ supported by GTEx data for %s tissue' % tiss,
            subgroups=dict(
            tissue=gtex_inv[tiss]))

        view_gtexSJ.add_tracks(track)  

    # Add track for all SJ supported by GTEx data (uniq SJ from set of all tissues)
    
    gtex_all_bed="gtex.all.bb"
    basename=os.path.basename(gtex_all_bed)
    gtex_all= Track(
            name="GTExSJ_All",
            tracktype='bigBed 4',
            url=url_base + basename,
            shortLabel='GTExSJ_All',
            longLabel='SJ supported by GTEx data for all tissues (unique SJ)',
            subgroups=dict(
            tissue="all"))
     
    view_gtexSJ.add_tracks(gtex_all)
    #print gtexSJ   

# ----------------------------------------------------------
# Building cell line tracks for ENCODE
# ----------------------------------------------------------
    encode_inv={} 
    
    for sg in encode_subgroups:
        if sg.name == "cell":
            for keys in sg.mapping:  
                encode_inv[sg.mapping[keys]]=keys

    encode_cell_file="/users/rg/buszczynska/Projects/sj.validation.180314/gen19.gtex.encode/gen19.encode/encode.cell.lines.txt"
    encode_cell=open(encode_cell_file, 'r')
    for cell_tmp in encode_cell:
        cell=cell_tmp.split('\n')[0]
        #print cell
        encode_bed="encode."+cell+".8.stg.reads.bb"
        basename=os.path.basename(encode_bed)
        track = Track(
            name="EncodeSJ"+"_"+cell,
            tracktype='bigBed 4',
            url=url_base + basename,
            shortLabel='EncodeSJ %s' % cell,
            longLabel='SJ supported by ENCODE data for %s cell line' % cell,
            subgroups=dict(
            cell=encode_inv[cell]))

        view_encodeSJ.add_tracks(track)

    # Add track for all SJ supported by GTEx data (uniq SJ from set of all tissues)
    
    encode_all_bed="encode.all.bb"
    basename=os.path.basename(encode_all_bed)
    encode_all= Track(
            name="EncodeSJ_All",
            tracktype='bigBed 4',
            url=url_base + basename,
            shortLabel='EncodeSJ_All',
            longLabel='SJ supported by ENCODE data for all cell lines (unique SJ)',
            subgroups=dict(
            cell="all"))
     
    view_encodeSJ.add_tracks(encode_all)
 
    #print encodeSJ

# ----------------------------------------------------------
# ADDING ALL
# ----------------------------------------------------------


    # Adding my track to trackDb (You need to do trackdb = TrackDb() earlier, because python will complain)
    trackdb.add_tracks(gtexSJ)
    trackdb.add_tracks(encodeSJ)
    trackdb.add_tracks(expValSJ)
    trackdb.add_tracks(gtexEncodeSJ)
    trackdb.add_tracks(allSJ) 

    # Bottom-up approach
    genome.add_trackdb(trackdb)
    genomes_file.add_genome(genome)
    hub.add_genomes_file(genomes_file)

    # Setting file names
    hub.local_fn = '/users/rg/buszczynska/public_html/gencode/track.hub/suppSJ/hub.txt'
    genomes_file.local_fn= '/users/rg/buszczynska/public_html/gencode/track.hub/suppSJ/genomes.txt'
    trackdb.local_fn= '/users/rg/buszczynska/public_html/gencode/track.hub/suppSJ/trackDb.txt'
  
# ----------------------------------------------------------
# RENDER and UPLOAD
# ----------------------------------------------------------
    
    results = hub.render()
    show_rendered_files(results)
    
    #for track in trackdb.tracks:
    #    upload_track(track=track, host="ant-login.linux.crg.es", user="buszczynska")

    #upload_hub(hub=hub, host="ant-login.linux.crg.es", user="buszczynska")



makeTrackHub()
