from trackhub import * 
import os
from trackhub.helpers import show_rendered_files
from trackhub.upload import upload_track, upload_hub

def makeTrackHub():

    url_base = 'http://genome.crg.es/~buszczynska/gencode/track.hub.valentina/data.files/'
    upload_base = '/users/rg/buszczynska/public_html/gencode/track.hub.valentina/data.files/'

# -----------------------------------------------------------
# HUB.TXT and GENOME FILE
# -----------------------------------------------------------

    hub = Hub(
        hub='breast.cancer',
        short_label='Breast cancer samples',
        long_label='Breast cancer healthy and tumor samples (Illumina HiSeq)',
        email='barbara.uszczynska@crg.es')

    genomes_file = GenomesFile()
    genome = Genome('hg19')
    trackdb = TrackDb()
    
# -----------------------------------------------------------
# Healthy SJ 
# -----------------------------------------------------------

    healthy = CompositeTrack(
        name="Healthy_samples",
        short_label="Healthy",
        long_label="Healthy samples",
        tracktype="bigWig 0 1000")
   
    healthy_subgroups=[
 
    SubGroupDefinition(
        name="sample",
        label="Sample",
        mapping=dict(BC125S="BC125S", SA4="SA4", BC22S="BC22S", BC47S="BC47S", BC45S="BC45S", SA1="SA1", BC174S="BC174S", BC118S="BC118S", SA2="SA2", BC103S="BC103S", BC32S="BC32S", SA3="SA3", BC112S="BC112S", BC15S="BC15S", BC40S="BC40S", BC74S="BC74S", BC62S="BC62S"))]

    healthy.add_subgroups(healthy_subgroups)
    healthy.add_params(color='13,181,119')
    

    # healthy View
    view_healthy = ViewTrack(
        name="Healthy",
        visibility="full",
        view="Signal",
        tracktype="bigWig 0 1000",
        short_label="Healthy",
        long_label="Healthy samples")

    healthy.add_view(view_healthy)
    
# -----------------------------------------------------------
# Luminal A
# -----------------------------------------------------------

    luminalA = CompositeTrack(
        name="LuminalA",
        short_label="LuminalA",
        long_label="Luminal A samples",
        tracktype="bigWig 0 1000")
       
    luminalA_subgroups=[
 
    SubGroupDefinition(
        name="samples",
        label="Samples",
        mapping=dict(BC71="BC71", M67="M67", BJ="BJ", BC64T="BC64T", M32="M32",
M92="M92", BC111="BC111", BC28T="BC28T", M8="M8", M34="M34", BC15T="BC15T", M78="M78", BC76="BC76", BC62T="BC62T", BC47T="BC47T", M75="M75", BC45T="BC45T", BC70T="BC70T", BC22T="BC22T"))]

    luminalA.add_subgroups(luminalA_subgroups) 
    luminalA.add_params(color='116,189,16')


    # LuminalA_View
    view_luminalA = ViewTrack(
        name="luminalA_View",
        visibility="full",
        view="Signal",
        tracktype="bigwig 0 1000",
        short_label="luminalA_View",
        long_label="Luminal A samples")

    luminalA.add_view(view_luminalA)

# -----------------------------------------------------------
# Luminal B
# -----------------------------------------------------------

    luminalB = CompositeTrack(
        name="LuminalB",
        short_label="LuminalB",
        long_label="Luminal B samples",
        tracktype="bigWig 0 1000")
       
    luminalB_subgroups=[
 
    SubGroupDefinition(
        name="samples",
        label="Samples",
        mapping=dict(BC174T="BC174T", BC1T="BC1T", BC78="BC78", BC75="BC75", BC112T="BC112T",BC118T="BC118T", BC103T="BC103T", BC125T="BC125T", BC77="BC77", M34="M34", BC169="BC169"))]

    luminalB.add_subgroups(luminalB_subgroups) 
    luminalB.add_params(color='116,189,16')


    # LuminalB_View
    view_luminalB = ViewTrack(
        name="luminalB_View",
        visibility="full",
        view="Signal",
        tracktype="bigwig 0 1000",
        short_label="luminalB_View",
        long_label="Luminal B samples")

    luminalB.add_view(view_luminalB)

# -----------------------------------------------------------
# TNBC
# -----------------------------------------------------------

    TNBC = CompositeTrack(
        name="TNBC",
        short_label="TNBC",
        long_label="TNBC samples",
        tracktype="bigWig 0 1000")
       
    TNBC_subgroups=[
 
    SubGroupDefinition(
        name="samples",
        label="Samples",
        mapping=dict(M40="M40", BC40T="BC40T", M26="M26", M125="M125", BC140T="BC140T",M149="M149", M53="M53", M100="M100", BC32TUM="BC32TUM", M158="M158", BC90T="BC90T", BC18="BC18", M153="M153", M156="M156", BC21="BC21"))]

    TNBC.add_subgroups(TNBC_subgroups) 
    TNBC.add_params(color='116,189,16')


    # LuminalB_View
    view_TNBC = ViewTrack(
        name="TNBC_View",
        visibility="full",
        view="Signal",
        tracktype="bigwig 0 1000",
        short_label="TNBC_View",
        long_label="TNBC samples")

    TNBC.add_view(view_TNBC)

# -----------------------------------------------------------
# TNBC
# -----------------------------------------------------------

    TNBC = CompositeTrack(
        name="TNBC",
        short_label="TNBC",
        long_label="TNBC samples",
        tracktype="bigWig 0 1000")
       
    TNBC_subgroups=[
 
    SubGroupDefinition(
        name="samples",
        label="Samples",
        mapping=dict(M40="M40", BC40T="BC40T", M26="M26", M125="M125", BC140T="BC140T",M149="M149", M53="M53", M100="M100", BC32TUM="BC32TUM", M158="M158", BC90T="BC90T", BC18="BC18", M153="M153", M156="M156", BC21="BC21"))]

    TNBC.add_subgroups(TNBC_subgroups) 
    TNBC.add_params(color='116,189,16')


    # LuminalB_View
    view_TNBC = ViewTrack(
        name="TNBC_View",
        visibility="full",
        view="Signal",
        tracktype="bigwig 0 1000",
        short_label="TNBC_View",
        long_label="TNBC samples")

    TNBC.add_view(view_TNBC)
   
# -----------------------------------------------------------
# EXPERIMENTAL VALIDATION
# -----------------------------------------------------------

    exp_bed="exp.validated.bb"
    basename=os.path.basename(exp_bed)
    expValSJ = Track(
        name="ExpValSJ",
        tracktype="bigBed 4",
        url=url_base + basename,
        visibility="squish",        
        short_label="ExpValSJ",
        long_label="SJ validated by RT-PCR-SEQ pipeline")

    expValSJ.add_params(color='234,194,92')

# -----------------------------------------------------------
# GTEX_ENCODE
# -----------------------------------------------------------

    gtex_encode_bed="gtex.encode.all.bb"
    basename=os.path.basename(gtex_encode_bed)
    gtexEncodeSJ = Track(
        name="GTExEncodeSJ",
        tracktype="bigBed 4",
        url=url_base + basename,
        visibility="squish",        
        short_label="GTExEncodeSJ",
        long_label="SJ supported by GTEx or ENCODE")

    gtexEncodeSJ.add_params(color='234,92,168')
# -----------------------------------------------------------
# ALL (GTEX/ENCODE/EXPVAL)
# -----------------------------------------------------------

    all_bed="all.bb"
    basename=os.path.basename(all_bed)
    allSJ = Track(
        name="allSuppSJ",
        tracktype="bigBed 4",
        url=url_base + basename,
        visibility="squish",        
        short_label="allSuppSJ",
        long_label="SJ experimentaly validated or are supported by GTEx/ENCODE data")

    allSJ.add_params(color='89,16,189')
 
# ----------------------------------------------------------
# Building tissue tracks for GTEx
# ----------------------------------------------------------
    gtex_inv={}
 
    for sg in gtex_subgroups:
        if sg.name == "tissue":
            for key in sg.mapping:
                gtex_inv[sg.mapping[key]]=key       
    
    gtex_tiss_file="/users/rg/buszczynska/Projects/sj.validation.180314/gen19.gtex.encode/gen19.gtex/gtex.tissues.txt"
 
    gtex_tiss=open(gtex_tiss_file, 'r')
    for tiss_tmp in gtex_tiss:
        tiss=tiss_tmp.split('\n')[0]
        gtex_bed="gtex."+tiss+".8.stg.reads.bb"
        basename=os.path.basename(gtex_bed)
        track = Track(
            name="GTExSJ"+"_"+tiss,
            tracktype='bigBed 4',
            url=url_base + basename,
            shortLabel='GTExSJ %s' % tiss,
            longLabel='SJ supported by GTEx data for %s tissue' % tiss,
            subgroups=dict(
            tissue=gtex_inv[tiss]))

        view_gtexSJ.add_tracks(track)  

    # Add track for all SJ supported by GTEx data (uniq SJ from set of all tissues)
    
    gtex_all_bed="gtex.all.bb"
    basename=os.path.basename(gtex_all_bed)
    gtex_all= Track(
            name="GTExSJ_All",
            tracktype='bigBed 4',
            url=url_base + basename,
            shortLabel='GTExSJ_All',
            longLabel='SJ supported by GTEx data for all tissues (unique SJ)',
            subgroups=dict(
            tissue="all"))
     
    view_gtexSJ.add_tracks(gtex_all)
    #print gtexSJ   

# ----------------------------------------------------------
# Building cell line tracks for ENCODE
# ----------------------------------------------------------
    encode_inv={} 
    
    for sg in encode_subgroups:
        if sg.name == "cell":
            for keys in sg.mapping:  
                encode_inv[sg.mapping[keys]]=keys

    encode_cell_file="/users/rg/buszczynska/Projects/sj.validation.180314/gen19.gtex.encode/gen19.encode/encode.cell.lines.txt"
    encode_cell=open(encode_cell_file, 'r')
    for cell_tmp in encode_cell:
        cell=cell_tmp.split('\n')[0]
        #print cell
        encode_bed="encode."+cell+".8.stg.reads.bb"
        basename=os.path.basename(encode_bed)
        track = Track(
            name="EncodeSJ"+"_"+cell,
            tracktype='bigBed 4',
            url=url_base + basename,
            shortLabel='EncodeSJ %s' % cell,
            longLabel='SJ supported by ENCODE data for %s cell line' % cell,
            subgroups=dict(
            cell=encode_inv[cell]))

        view_encodeSJ.add_tracks(track)

    # Add track for all SJ supported by GTEx data (uniq SJ from set of all tissues)
    
    encode_all_bed="encode.all.bb"
    basename=os.path.basename(encode_all_bed)
    encode_all= Track(
            name="EncodeSJ_All",
            tracktype='bigBed 4',
            url=url_base + basename,
            shortLabel='EncodeSJ_All',
            longLabel='SJ supported by ENCODE data for all cell lines (unique SJ)',
            subgroups=dict(
            cell="all"))
     
    view_encodeSJ.add_tracks(encode_all)
 
    #print encodeSJ

# ----------------------------------------------------------
# ADDING ALL
# ----------------------------------------------------------


    # Adding my track to trackDb (You need to do trackdb = TrackDb() earlier, because python will complain)
    trackdb.add_tracks(gtexSJ)
    trackdb.add_tracks(encodeSJ)
    trackdb.add_tracks(expValSJ)
    trackdb.add_tracks(gtexEncodeSJ)
    trackdb.add_tracks(allSJ) 

    # Bottom-up approach
    genome.add_trackdb(trackdb)
    genomes_file.add_genome(genome)
    hub.add_genomes_file(genomes_file)

    # Setting file names
    hub.local_fn = '/users/rg/buszczynska/public_html/gencode/track.hub/suppSJ/hub.txt'
    genomes_file.local_fn= '/users/rg/buszczynska/public_html/gencode/track.hub/suppSJ/genomes.txt'
    trackdb.local_fn= '/users/rg/buszczynska/public_html/gencode/track.hub/suppSJ/trackDb.txt'
  
# ----------------------------------------------------------
# RENDER and UPLOAD
# ----------------------------------------------------------
    
    results = hub.render()
    show_rendered_files(results)
    
    #for track in trackdb.tracks:
    #    upload_track(track=track, host="ant-login.linux.crg.es", user="buszczynska")

    #upload_hub(hub=hub, host="ant-login.linux.crg.es", user="buszczynska")



makeTrackHub()
