#!/usr/bin/python

__author__="BU"

from optparse import OptionParser
import gzip
from os import sys
import getopt

# ===============
## OPTION PARSING
# ===============
parser = OptionParser()
parser.add_option("-i", 
	help="The input format is gff with uniprot terms. No stdin")
parser.add_option('-o', help='Output file with exon location in respect to given uniprot term')
parser.add_option('-r',help='Tab-separated file containing principal isoform id and reference exons: chr_start_stop_strand. No stdin')

options, args = parser.parse_args()


# Store input and output file

infile=''
outfile=''
ref=''

# Read command line args
myopts, args = getopt.getopt(sys.argv[1:],"i:o:r:")

# Loop over arguments and assign input to infile and output to outfile 
# o == option
# a == argument passed to the o

if myopts == []:
    print("Usage: %s -i input -o output -r ref" % sys.argv[0])

for o, a in myopts:
    if o == '-i':
        infile=a
    if o == '-o':
        outfile=a
    if o == '-r':
        ref=a

print ("Input file : %s \nOutput file: %s \nReference: %s" % (infile,outfile,ref))

def select():

    gene_uniprot={} 
    pi_exon={}
    locate={}
    control=[]

    gtf_file=open(infile, 'r')
    for line in gtf_file:
        line_split=line.split("\n")[0].split("\t")
        domain=line_split[1]
        typ=line_split[2]

        if typ == "exon":
      
            start=line_split[3]
            stop=line_split[4]
            strand=line_split[6]
            txID=line_split[8].split(';')[1].split('transcript_id')[1]
            geneID=line_split[8].split(';')[0].split('gene_id')[1].split('"')[1]
            record=domain+"@"+start+"@"+stop+"@"+txID.lstrip()
 
            if geneID not in gene_uniprot:
                gene_uniprot[geneID]=[record]
            else:
                gene_uniprot[geneID].append(record)
            #print geneID, gene_uniprot[geneID]
 
    reference_file=open(ref, 'r')
    for lin in reference_file:
        lin_split=lin.split('\n')[0].split('\t')
        pi=lin_split[0]
        exon_ref=lin_split[1]
 
        if pi not in pi_exon:
            pi_exon[pi]=[exon_ref]
        else:
            pi_exon[pi].append(exon_ref)
        #print pi, pi_exon[pi]  
    
    for key in pi_exon:
        if key in gene_uniprot:
            control.append(key)
            for item in pi_exon[key]:
                #print 'ITEM', item
                start_ref=int(item.split('_')[1])
                #print 'START_REF', start_ref
                stop_ref=int(item.split('_')[2])
                #print 'STOP_REF', stop_ref
                strand=item.split('_')[3]
                #print 'STRAND', strand
                length=stop_ref - start_ref
                for value in gene_uniprot[key]:
                    uni_domain=value.split('@')[0]
                    uni_start=int(value.split('@')[1])
                    uni_stop=int(value.split('@')[2])
                    uni_tx=value.split('@')[3]

                    if strand == "+":
                     
                        if uni_start < start_ref and uni_stop < start_ref: 
                            take=key+"\t"+item+"\t"+uni_domain+"\t"+"UPSTREAM"+"\t"+uni_tx+"\t"+"UP"
  
                        if uni_start > stop_ref:
                            take=key+"\t"+item+"\t"+uni_domain+"\t"+"DOWNSTREAM"+"\t"+uni_tx+"\t"+"DWN"  
               
                        if uni_start <= start_ref and uni_stop >= start_ref:
                            take=key+"\t"+item+"\t"+uni_domain+"\t"+"OVERLAPPING"+"\t"+uni_tx+"\t"+"1"

                        if uni_start >= start_ref and stop_ref >= uni_start:
                            take=key+"\t"+item+"\t"+uni_domain+"\t"+"OVERLAPPING"+"\t"+uni_tx+"\t"+"2"

                        if uni_start <= start_ref and stop_ref <= uni_stop:
                            take=key+"\t"+item+"\t"+uni_domain+"\t"+"OVERLAPPING"+"\t"+uni_tx+"\t"+"3"  
               
                        if uni_start >= start_ref and stop_ref >= uni_stop:
                            take=key+"\t"+item+"\t"+uni_domain+"\t"+"OVERLAPPING"+"\t"+uni_tx+"\t"+"4" 
                        
                             
                        if key not in locate:
                            locate[key]=[take]
                        else:
                            locate[key].append(take)
  
                    if strand == "-":
                     
                        if uni_start < start_ref and uni_stop < start_ref: 
                            take=key+"\t"+item+"\t"+uni_domain+"\t"+"DOWNSTREAM"+"\t"+uni_tx #+"\t"+"DWN"

                        if uni_start > stop_ref:
                            take=key+"\t"+item+"\t"+uni_domain+"\t"+"UPSTREAM"+"\t"+uni_tx+"\t"+"UP"                        

                        if uni_start <= start_ref and uni_stop >= start_ref:
                            take=key+"\t"+item+"\t"+uni_domain+"\t"+"OVERLAPPING"+"\t"+uni_tx+"\t"+"1"

                        if uni_start >= start_ref and stop_ref >= uni_start:
                            take=key+"\t"+item+"\t"+uni_domain+"\t"+"OVERLAPPING"+"\t"+uni_tx+"\t"+"2"

                        if uni_start <= start_ref and stop_ref <= uni_stop:
                            take=key+"\t"+item+"\t"+uni_domain+"\t"+"OVERLAPPING"+"\t"+uni_tx+"\t"+"3"  
               
                        if uni_start >= start_ref and stop_ref >= uni_stop:
                            take=key+"\t"+item+"\t"+uni_domain+"\t"+"OVERLAPPING"+"\t"+uni_tx+"\t"+"4" 
     
  
                        if key not in locate:
                            locate[key]=[take]
                        else:
                            locate[key].append(take)

    #print 'LENGTH_CONTROL', len(set(control))

    t=open("CONTROL.LIST", 'w')
    for i in set(control):
        t.write('%s\n' % i)
    t.close()


    f=open(outfile,'w')
    for kyz in locate:
        for v in locate[kyz]:
            f.write('%s\n' % v)
    f.close()

select()  
             
                  
